#pragma rtGlobals=1		// Use modern global access method.
#pragma version=1	.0	// to help with version control

//#include "2008GeneralMacros"			//  should reside somewhere in the Igor Pro Folder/User Procedures folder

// *************************************************
//	2008ICARTTFormat.ipf
//	Author:Donna Sueper
//  Revision Number: 1.00
//	Revision Date: Jan 10, 2008
//	Revision Notes:  

//		Same as 2006 but globals updated to 2008 settings (ARCPAC, ICEALOT) etc.
//
// *************************************************
// These functions create and load data files of the format indicated in the 2008 ICARTT Data
// Management Plan document, which was roughly based on the Haines-Hipskind Nasa Ames Format.
// Functions and macros herein are not appropriate for all
// data generated by ICARTT participants; assumptions are outlined below.
//
// Functions in this file use functions in the 2008 GeneralMacros (latest version) igor procedure file.
// Users should have both files open in an igor experiement.
//
// Whenever a user loads or creates a ICARTT format file, an igor data folder called
// ICARTTFileFormat gets created in the experiment and most of the work is done here.
// If something goes awry, the user can reset the data folder to root (most everyone's default)
// by inserting the text below (less the //) into the command line
// SetDataFolder root:
//
// Some paramters are calculated; some have simple default values.  They are:
// NumHeaderLines - gets calculated
// FileFormatType = 1001 // this value is assumed for reading and creating files.
// FileVolumeNumber = 1 // this value is assumed for reading and creating files.
// TotalVolumeNumber = 1 // this value is assumed for reading and creating files.
// UTCDataStart - set from selected time wave
// NumVariables - gets set when user selects waves
// ScaleFactor = 1 // this value is assumed for reading and creating files.
// MissingDataIndicator = -9999 // NOTE - The same missing value is used for all parameters when creating a file.
// NumSpecialCommentLines - gets calculated
// NumNormalCommentLines - gets calculated
// Normal Comments are the concatenation of several other parameters
//
// None of the original waves in the experiment are modified in any way.
// Waves selected to be saved are duplicated in the ICARTTFileFormat data folder.  
// The UTC timewave (the independent parameter) and any dependent parameters formatted with a "t" (from the create panel)
// will have the starting date subtracted from it (in the created file) and will be formatted with one digit after the decimal.
// 
// When loading an icartt file into an experiment, the missing value as reported in the header will be used
// to convert missing values in the incoming waves to nans.  The independent variable (time) will have the
// UTC date (as indicated in the header) added to it
// 
// When loading data from the LoadDataOnly menu item, almost no format checking is done and missing data isn't changed to nans.
//
// The name of the wave in the experiment will be the name given to the dependent and independent parameters, with the
// exception of matricies, where column names much as mymatrix0, mymatrix1, etc are used.
//
//  If anything is named in the experiment in a 'liberal' fashion (i.e. wave names with spaces), all bets are off.
//
//  Users should be aware that there are opearting system differences for end of line designations.
//  Mac uses carriage return (CR), Unix uses line feed (LF), and Windows use carriage return + line feed (CRLF)
//  For icarrtt, the preferred end of line designation is CRLF (which correspondes to \r\n in the code). 
// *************************************************

// *************************************************
Menu "ICARTTFormat"
	"-"
	"CreateICARTT1001"
	"LoadICARTT1001"
	"LoadICARTTDataOnly1001"
	"-"
End


// *************************************************
// Creates all the global waves, variables and strings all the other functions need execpt those specific to the icartt panel.
// Sets default values for many of the parameters.
// This function should be called before any loading or creating of ICARTT files is done.

Function Icartt_CreateGlobals()

	string  getFolderStr

	getFolderStr = GetDataFolder(1)
	
	string MyWaveList = WaveList("*", ";", "")
	variable n = ItemsInList(MyWaveList)
	string currentTimeStr = time2yyyymmddhhmmss(DateTime)
	
	NewDataFolder/O/S root:ICARTTFileFormat

	string/g MyWaveListStr = MyWaveList
	string/g RadioValStr = ""
	string/g FileNameStr = "DataID_NP3_2008mmdd_RA.ict"
	string/g FileNameStr4Macs = ""  // igor for Mac does not handle long file names gracefully. 
	string/g MergedSuffixStr = ""  // is "All1SecMerged" or some such for merged files

	variable/g NumHeaderLinesVar
	string/g FileFormatTypeStr = "1001"

	string/g PINameStr
	string/g PIAffiliationStr = "NOAA Chemical Sciences Division"
	string/g DataSourceDescriptionStr = "xx Instrument "
	string/g MissionNameStr = "ARCPAC 2008"
	variable/g FileVolumeNumberVar = 1
	variable/g TotalVolumeNumberVar = 1

	string /g UTCDataStartStr		// has spaces!
	variable/g/d UTCDataStartVar
	string/g UTCRevisionDateStr = currentTimeStr[0,3]+ " "+ currentTimeStr[4,5]+ " "+ currentTimeStr[6,7] // has spaces!
	variable/g DataIntervalVar = 1

	string/g IndependentVariableNameStr
	variable/g NumVariablesVar
	variable/g ScaleFactorVar = 1
	variable/g MissingDataIndicatorVar = -9999
	string/g VariableNamesList

	variable/g NumSpecialCommentLinesVar
	string/g SpecialCommentsStr

	variable/g NumNormalCommentLinesVar
	string/g NormalCommentsStr
	
	string/g PIContactInfoStr = "Address: NOAA Chemical Sciences Division, Mail Stop ESRL/CSD7, 325 Broadway, Boulder, CO 80305"
	string/g PlatformSiteInfoStr = "NOAA WP-3D / NOAA RV Ronald H. Brown"
	string/g LocationInfoStr = "The aircraft/ship was stationed in. Location data is in AircraftPos/RHBPos file."
	string/g AssociatedFileNamesStr = "N/A"
	string/g InstrumentInfoStr = "N/A"
	string/g DataInfoStr = "Units:"
	string/g UncertaintyInfoStr = "N/A"

	variable/g ULODFlagVar = -7777
	string/g ULODValueStr = "N/A"
	variable/g LLODFlagVar = -8888
	string/g LLODValueStr = "N/A"

	string/g DataManagerContactInfoStr = "Data Manager:Ken Aikin; kenneth.c.aikin@noaa.gov"
	string/g ProjectInfoStr = "ARCPAC / ICEALOT   Dates of Study."
	string/g StipulationsOnUseStr = "Use of these data require prior ok from PI."

	string/g OtherCommentsStr = "N/A"
	variable/g NumOtherCommentsVar

	variable/g RevisionNumberVar = 0		// The original, first file  is revision # 0 - will be more useful at a later time
	string/g RevisionNumberStr = "A"		// prelim data designation
	string/g RevisionStr = "RA: Preliminary Data.  For in-field use only."
		
	variable/g FileOpenAfterCreateVar
	variable/g FormatAsInTopTableVar
	variable/g IsMatrixData
	string/g MatrixColNameList

	variable/g ExponFormatVar = 5   // the default number of digits to be saved when using the exponential format

	string/g MyTempStr			// for reordering waves in list
	make/o/t/n=0 TempTextWave			// for reordering waves in list
		
	string myparamlist=""	
	myparamlist += "FileName;NumHeaderLines_FileType;PIName;PIAffiliation;DataSourceDescription;MissionName;FileVolNum_TotalVolNum;UTCDataStart_RevDate;DataInterval;"
	myparamlist += "IndependentVariableName;NumVariables;ScaleFactor;MissingDataIndicator;VariableNames;NumSpecialCommentLines;SpecialComments;"
	myparamlist += "NumNormalCommentLines;NormalComments;PIContactInfo;PlatformSiteInfo;LocationInfo;AssociatedFileNames;InstrumentInfo;DataInfo;"
	myparamlist +=   "Uncertainty;ULODFlag;ULODValue;LLODFlag;LLODValue;DMContactInfo;ProjectInfo;StipulationsOnUse;OtherComments;RevisionNumber;Rev;"

	make/o/t/n=(ItemsInList(myparamlist)) ICARTTHeaderParam, ICARTTHeaderValue
	List2TextWaveNoCreate(myparamlist, ICARTTHeaderParam)
	
	if (n==0)
		make/t/o/n=(0) WaveListTextWave		// for ListBox 
		make/t/o/n=(0,2) WaveListTextWaveMatrix		//  ListBox; 1st Column is # of decimal place, 2nd is for names
		make/o/n=(0,2) SelWaveMatrix		// used with ListBox
	else
		make/t/o/n=(n-1) WaveListTextWave		// for ListBox 
		make/t/o/n=(n-1,2) WaveListTextWaveMatrix		//  ListBox; 1st Column is # of decimal place, 2nd is for names
		make/o/n=(n-1,2) SelWaveMatrix		// used with ListBox
	endif
	
	WaveListTextWaveMatrix[][0] = "" // the default number of digits after the decimal
	WaveListTextWaveMatrix[][1] = ""
	
	SelWaveMatrix[][0] = 2		// First column, containing number of digits past decimal, will be editable
	SelWaveMatrix[][1] = 0
		
	SetDataFolder getFolderStr
	
End


// *************************************************
// Makes sure that the format for UTCDateStr is or can be interpretted nicely to yyyy mm dd.
// Also makes sure time is reasonable, after 1980, an arbitrary default.

Function Icartt_CheckUTCDateStr(UTCDateStr)
	String UTCDateStr

	variable UTCTime
	string tempDateStr
	
	tempDateStr =   RemoveDoubleSpaces(RemoveCarriageReturns(UTCDateStr))
	tempDateStr = Icartt_TeaseOutDate(tempDateStr, 0)
	
	if (strlen(tempDateStr)!=10)
		Abort "The string length of the time "+UTCDateStr+" should be10 (yyyy mm dd) - Aborting from Icartt_CheckUTCDateStr"
	endif
	
	UTCTime =  yyyymmddhhmmss2time(tempDateStr[0,3]+tempDateStr[4,5]+tempDateStr[6,7] + "000000")
	if ( UTCTime < date2secs(1980,1,1)  )
		Abort "The string "+UTCDateStr+"indicates a time before 1980; is something wrong? - Aborting from Icartt_CheckUTCDateStr"
	endif

End


// *************************************************
// Checks the revision portion of the file name and sets the global variable RevisionNumberVar.
// Revisions in field data, with an RA, RB, etc. are treated as revisions 0, 1, etc.

Function Icartt_CheckRevStrSetNumber()

	SVAR FileNameStr = root:ICARTTFileFormat:FileNameStr
	NVAR RevisionNumberVar = root:ICARTTFileFormat:RevisionNumberVar
	
	string fileNameRevStr 
	variable pos1, pos2, pos3, pos4, pos5, n
	
	//  assumes file name of the form DataID_platform_yyyymmdd_Rx_mergeorother.ict
	
	pos1 = strsearch(FileNameStr, "_", 0 )				// identifies DataID_platform
	pos2 = strsearch(FileNameStr, "_", pos1+1)		// identifies platform_yyyymmdd
	pos3 = strsearch(FileNameStr, "_", pos2+1)		// identifies yyyymmdd_Rx
	pos4 = strsearch(FileNameStr, "_", pos3+1)		// identifies Rx_mergeorother if it exists
	pos5 = strsearch(FileNameStr, ".", 0)				// identifies.ict
	
	if (cmpstr(lowerstr(FileNameStr[pos3+1]), "r")!=0  )
		Abort "There isn't a \"_Rnumber\" in the file name; it was expected after "+FileNameStr[0, pos3]+".  Please check the file name. - Aborting from Icartt_CheckRevStrSetNumber"
	endif

	fileNameRevStr=FileNameStr[pos3+1, max(pos4, pos5)-1 ]			// assumes revision numbers are only one character
	
	if (strlen(fileNameRevStr)<=0)
		Abort "There isn't a \"_R#\" in the file name; saw "+fileNameRevStr + " instead - Aborting from Icartt_CheckRevStrSetNumber"
	endif
	
	n = str2num(FileNameRevStr[1])
	if (numtype(n)!=0)
		if (cmpstr(lowerstr(FileNameRevStr[1]), "a") ==0)
			RevisionNumberVar=0
		elseif(cmpstr(lowerstr(FileNameRevStr[1]), "b") ==0)
			RevisionNumberVar=1
				elseif(cmpstr(lowerstr(FileNameRevStr[1]), "c") ==0)
			RevisionNumberVar=2
		elseif(cmpstr(lowerstr(FileNameRevStr[1]), "d") ==0)
			RevisionNumberVar=3		
		endif
	else
		RevisionNumberVar	= n
	endif
	
End

	
// *************************************************
// Checks the DataIntervalVar and the values in the timewave match.

Function Icartt_CheckIndependentVarTime()

	NVAR DataIntervalVar =root:ICARTTFileFormat:DataIntervalVar
	SVAR IndependentVariableNameStr =root:ICARTTFileFormat:IndependentVariableNameStr
	SVAR MergedSuffixStr =root:ICARTTFileFormat:MergedSuffixStr

	WAVE indepWave = $IndependentVariableNameStr
	
	String  getFolderStr
	
	getFolderStr = GetDataFolder(1)

	if (WaveExists(indepwave) ==0)
		Abort "The independent parameter wave "+IndependentVariableNameStr+" does not seem to exist - Aborting from Icartt_CheckIndependentVarTime"
	endif
	
	SetDataFolder root:ICARTTFileFormat		// set the data folder to here because we are going to do some wave creation.
 	
 	Duplicate/o indepWave, timeDiff
	timeDiff = indepWave[p] - indepWave[p-1]
	timeDiff[0] = nan
	Wavestats/q timeDiff
	
	Killwaves/Z timeDiff	

	SetDataFolder getFolderStr

	 if   ( (DataIntervalVar!=0) && (strlen(MergedSuffixStr)==0)  ) // Not start -stop type data, not merged file	
		if ( (numpnts(indepWave)!=1) && ((DataIntervalVar !=V_min)  || (DataIntervalVar !=V_max)) )
			Abort "The data interval and the difference between successive values in the independent wave don't match -  Aborting from Icartt_CheckIndependentVarTime"
		endif	
	else  // We have start-stop data or a merged file  Just check for monotonically increasing.
		if ( V_min <=0 )
			Abort "Please check to see if the timewave has duplicate or non-increasing values. - Aborting from Icartt_CheckIndependentVarTime"
		endif
	endif
		
End


// *************************************************
// Checks to see if the date in the filename matched the UTCDataStartVar parameter.

Function Icartt_CheckHeaderDate()
		
	NVAR UTCDataStartVar = root:ICARTTFileFormat:UTCDataStartVar
	SVAR UTCDataStartStr = root:ICARTTFileFormat:UTCDataStartStr
	SVAR FileNameStr= root:ICARTTFileFormat:FileNameStr

	string str
	variable strTime, pos1, pos2, pos3, pos4, pos5
		
	//  assumes file name of the form DataID_platform_yyyymmdd_Rx_possibleMergeOrOther.ict
	
	pos1 = strsearch(FileNameStr, "_", 0 )				// identifies DataID_platform
	pos2 = strsearch(FileNameStr, "_", pos1+1)		// identifies platform_yyyymmdd
	pos3 = strsearch(FileNameStr, "_", pos2+1)		// identifies yyyymmdd_Rx
	pos4 = strsearch(FileNameStr, "_", pos3+1)		// identifies Rx_mergeorother if it exists
	pos5 = strsearch(FileNameStr, ".", 0)				// identifies.ict

	str = FileNameStr[pos2+1, pos3-1]	// UTCDataStart
	
	strTime = yyyymmddhhmmss2time(str+"000000")

	if (strTime!=UTCDataStartVar)
		Abort "The date in the file name "+str+" does not match the UTCDataStart value from the independent parameter "+StrSubstitute(" ", UTCDataStartStr ,"")+ " - Aborting from Icartt_CheckHeaderDate"
	endif

End


// *************************************************
// Checks to see if the date in the filename contains illegal chars and if we are dealing with a merged data set.

Function Icartt_CheckFileNameStr()
	
	SVAR FileNameStr= root:ICARTTFileFormat:FileNameStr
	SVAR FileNameStr4Macs= root:ICARTTFileFormat:FileNameStr4Macs
	SVAR MergedSuffixStr = root:ICARTTFileFormat:MergedSuffixStr

	string str, cmd
	variable pos1, pos2, pos3, pos4, pos5, i, j, n

	n = strlen(FileNameStr)
	if (n==0)
		Abort "Please enter a valid filename - Aborting from Icartt_CheckFileNameStr"
	endif
	
	for(i=0;i<n;i+=1)
		j = strsearch("abcdefghijklmnopqrstuvwxyz0123456789.-_", lowerStr(FileNameStr[i]), 0) 
		if (j<0)
			Abort "The filename "+FileNameStr+"contains "+FileNameStr[i]+" Only alphanumeric and .-_ are allowed - Aborting from Icartt_CheckFileNameStr"
		endif
	endfor

	if ( cmpstr(".ict", LowerStr(FileNameStr[strlen(FileNameStr)-4, strlen(FileNameStr)-1])) !=0 )
		Abort "The filename "+FileNameStr+"must end with a \".ict\" - Aborting from Icartt_CheckFileNameStr"
	endif

	//  assumes file name of the form DataID_platform_yyyymmdd_Rx_mergeorother.ict
	
	pos1 = strsearch(FileNameStr, "_", 0 )				// identifies DataID_platform
	pos2 = strsearch(FileNameStr, "_", pos1+1)		// identifies platform_yyyymmdd
	pos3 = strsearch(FileNameStr, "_", pos2+1)		// identifies yyyymmdd_Rx
	pos4 = strsearch(FileNameStr, "_", pos3+1)		// identifies Rx_mergeorother if it exists
	pos5 = strsearch(FileNameStr, ".", 0)				// identifies.ict

	str = FileNameStr[pos4+1, pos5-1]
	
	if (strlen(str)>0)
		MergedSuffixStr = str
	else
		MergedSuffixStr = ""
	endif

End

Function/S DateFromICARTTname(fname)	//Pick out date from icartt file name
	string fname
	
	string str
	variable  pos1, pos2, pos3
		
	//  assumes file name of the form DataID_platform_yyyymmdd_Rx_mergeorother.ict
	
	pos1 = strsearch(fname, "_", 0 )				// identifies DataID_platform
	pos2 = strsearch(fname, "_", pos1+1)		// identifies platform_yyyymmdd
	pos3 = strsearch(fname, "_", pos2+1)		// identifies yyyymmdd_Rx

	str = fname[pos2+1, pos3-1]	// UTCDataStart
	return str
End Function

// *************************************************
// Returns 1, true, if the string has the characters "time", "utc", "start", or "stop" or any capitalized variations.

Function Icartt_IsATimeString(str)
	String str

	if (strsearch(LowerStr(str), "time", 0) >=0)
		return 1
	elseif (strsearch(LowerStr(str), "utc", 0) >=0)
		return 1
	elseif (strsearch(LowerStr(str), "start", 0) >=0)
		return 1
	elseif (strsearch(LowerStr(str), "stop", 0) >=0)
		return 1
	elseif (strsearch(LowerStr(str), "secmid", 0) >=0)
		return 1
	elseif (strsearch(LowerStr(str), "minmid", 0) >=0)
		return 1
	else
		return 0
	endif

End

function GetOffsetToRevYear(datestr)	// pass in string containing file date and modification date. Return offset to start of string containing Modification year. ie. pass in "2006 9 20 2007 8 24", and return 10 (the position of 2007)
	string datestr

	variable numspaces, i, len, pos
	
	len = strlen(datestr)
	if (cmpstr((datestr[len-1]), " ")==0)	//remove any trailing spaces at end of date
		do
			datestr = RemoveEnding(datestr, " ")
			len = strlen(datestr)
//			print "trimming string. len= ", len
		while(cmpstr((datestr[len-1]), " ")==0)
	endif
	
	i=-1
	numspaces=0
	do
		i = strsearch(datestr, " ", i+1)
		if (i!=-1)
			numspaces+=1
		endif
	while(i!=-1)
	
	if (numspaces==5)	//date is correctly formatted
		for(i=0; i<3; i+=1)
			pos = strsearch(datestr, " ", pos+1)
		endfor
	endif
	
	return pos+1	//want starting character of year of modification
end


// *************************************************
// Sets globals FileNameStr and FileNameStr4Mac if user does not cancel dialog box; does NOT set IcarttFilePath.

Function Icartt_SelectFileDialog()

	SVAR FileNameStr = root:ICARTTFileFormat:FileNameStr
	SVAR FileNameStr4Macs = root:ICARTTFileFormat:FileNameStr4Macs
	SVAR IcarttPathStr = root:ICARTTFileFormat:Panel:icarttPathStr

	variable refNum
	string  pathStr
	
	Open/C="????"/T="????"/D/R/M= "Select an ICARTT file." refNum 		// /D is dialog only; we don't have to close the file because it is never opened

	if (strlen(s_filename)>0)
		// first deal with the path
		pathStr = FilePathOnly(s_filename)
		
		Newpath/q/o ICARTTFilePath pathStr		// creates or overwrites the path where the file is located.			
		IcarttPathStr =  ConvertPathStr("IcarttFilePath")		// for the panel
	
		// now deal with file name
		FileNameStr4Macs = s_filename[strlen(pathStr), strlen(s_filename)-1]

		if (cmpstr("Macintosh", IgorInfo(2))==0  )		// Macs can truncate filenames
			ExecuteScriptText/Z "get alias \""+s_filename+"\" as text"
			s_filename = S_Value[1, strlen(S_Value)-2]		// strip out beginning and ending quotes			
			FileNameStr = s_filename[strlen(pathStr), strlen(s_filename)-1]
		else
			FileNameStr = FileNameStr4Macs		
		endif
	
	else
		FileNameStr = ""		
		FileNameStr4Macs = ""		
	endif

End


// *************************************************
// Sets important globals as an initial step to see if an icartt file is valid.

Function Load_GetNumHeaderLinesFileTyp(fileName)
	String fileName

	NVAR NumHeaderLinesVar = root:ICARTTFileFormat:NumHeaderLinesVar
	SVAR FileFormatTypeStr = root:ICARTTFileFormat:FileFormatTypeStr
	NVAR NumVariablesVar = root:ICARTTFileFormat:NumVariablesVar
	
	variable refNum,y
	string lineStr
				
	Icartt_SetIcartFilePath()
	
	Open/R/Z=2/P=IcarttFilePath refNum as fileName
	
	if (refNum==0)
		Abort "The file "+filename+" was not opened - Aborting from Load_GetNumHeaderLinesFileTyp"//+num2str( V_flag)
	endif
				
	// Read and parse first line
	FReadLine refNum, lineStr

	numHeaderLinesVar = str2num(TeaseOutNumbers(lineStr, 0))
	
	if (numtype(numHeaderLinesVar)!=0)
		Abort "Had trouble interpretting the number of lines in the header from the first line in the file. - Aborting from Load_GetNumHeaderLinesFileTyp"
	endif	
		
	FileFormatTypeStr = TeaseOutNumbers(lineStr, strlen(TeaseOutNumbers(lineStr, 0)) + 1)
	FileFormatTypeStr = RemoveCarriageReturns(FileFormatTypeStr)

	for (y=0;y<9;y+=1)		// The number of primary variables should always be on the 10th line.
		FReadLine refNum, lineStr
	endfor
	
	NumVariablesVar = str2num(lineStr)
	Close refNum

	if (numtype(NumVariablesVar)!=0)
		Abort "Couldn't determine the number of variables in the file - Aborting from Load_GetNumHeaderLinesFileTyp"
	endif

End


// *************************************************
// Sets the UTCDataStartVar and UTCDataStartStr from the date in the filename.

Function Load_SetDateFromFilename()
		
	NVAR UTCDataStartVar = root:ICARTTFileFormat:UTCDataStartVar
	SVAR UTCDataStartStr = root:ICARTTFileFormat:UTCDataStartStr
	
	SVAR FileNameStr= root:ICARTTFileFormat:FileNameStr

	string str
	variable  pos1, pos2, pos3, pos4, pos5
		
	//  assumes file name of the form DataID_platform_yyyymmdd_Rx_mergeorother.ict
	
	pos1 = strsearch(FileNameStr, "_", 0 )				// identifies DataID_platform
	pos2 = strsearch(FileNameStr, "_", pos1+1)		// identifies platform_yyyymmdd
	pos3 = strsearch(FileNameStr, "_", pos2+1)		// identifies yyyymmdd_Rx
	pos4 = strsearch(FileNameStr, "_", pos3+1)		// identifies Rx_mergeorother if it exists
	pos5 = strsearch(FileNameStr, ".", 0)				// identifies.ict

	str = FileNameStr[pos2+1, pos3-1]	// UTCDataStart
	
	UTCDataStartVar = yyyymmddhhmmss2time(str+"000000")

	UTCDataStartStr = str[0,3]+" "+str[4,5]+ " "+ str[6,7]
	
End


// *************************************************
// Simply loads lines of text from a file; does little error checking.
// Load_IngestHeader1001 function does most error checking.

Function Load_LoadHeader1001(fileName)
	String fileName
		
	NVAR RevisionNumberVar = root:ICARTTFileFormat:RevisionNumberVar
	NVAR NumHeaderLinesVar = root:ICARTTFileFormat:NumHeaderLinesVar
		
	WAVE/t ICARTTHeaderParam = root:ICARTTFileFormat:ICARTTHeaderParam
	WAVE/t ICARTTHeaderValue = root:ICARTTFileFormat:ICARTTHeaderValue
 		
	variable refNum, i, y, numLinesHaveBeenRead
	string lineStr, str, str1
	variable numVariables, numSpecialCommentLinesVar, numNormalCommentLinesVar
		
	Load_GetNumHeaderLinesFileTyp(fileName)
	Icartt_CheckRevStrSetNumber()
	
 	Open/R/Z=2/P=IcarttFilePath refNum as fileName
				
	numLinesHaveBeenRead = 0
	
	// Read and parse first line
	ICARTTHeaderValue[0] = fileName// FileName

	FReadLine refNum, lineStr
	ICARTTHeaderValue[1] =  lineStr		// NumHeaderLines_FFI
	FReadLine refNum, lineStr
	ICARTTHeaderValue[2] = lineStr		// PIName
	FReadLine refNum, lineStr
	ICARTTHeaderValue[3] = lineStr		//PIAffiliation
	FReadLine refNum, lineStr
	ICARTTHeaderValue[4] = lineStr		//DataSourceDescription
	FReadLine refNum, lineStr
	ICARTTHeaderValue[5] = lineStr		//MissionName
	FReadLine refNum, lineStr
	ICARTTHeaderValue[6] = lineStr  	//FileVolumeNumber, TotalVolumeNumber
 	FReadLine refNum, lineStr
	ICARTTHeaderValue[7] = lineStr		//UTCRevisionDate
	FReadLine refNum, lineStr
	ICARTTHeaderValue[8] = lineStr		//DataInterval
	FReadLine refNum, lineStr
	ICARTTHeaderValue[9] = lineStr		//IndependentVariableName
	FReadLine refNum, lineStr
	ICARTTHeaderValue[10] = lineStr		//NumVariables

	numVariables = str2num(ICARTTHeaderValue[10]) //NumVariables
	
	if ((numVariables >400) || (numVariables <0) )		// currently the max number of dependent parameters is 400
		close refNum
		Abort "Had trouble reading in the number of dependent parameters.-  Aborting from Load_LoadHeader1001"
	endif	
	
 	FReadLine refNum, lineStr
	ICARTTHeaderValue[11] = lineStr		//ScaleFactor
	FReadLine refNum, lineStr
	ICARTTHeaderValue[12] = lineStr		//MissingDataIndicator

	numLinesHaveBeenRead = 12
	
	FReadLine refNum, lineStr
	if (numVariables>1)
		str=lineStr+ ";"
		for(i=0;i<numVariables-1; i+=1)
			FReadLine refNum, lineStr
			str+=lineStr + ";"
			numLinesHaveBeenRead+=1
		endfor
		ICARTTHeaderValue[13] = str	//	VariableNames	
	else
		ICARTTHeaderValue[13] = lineStr		//	VariableNames	
	endif

	FReadLine refNum, lineStr
	ICARTTHeaderValue[14] = lineStr		//	NumSpecialCommentLines	
	numLinesHaveBeenRead+=1
	
	numSpecialCommentLinesVar = str2num(ICARTTHeaderValue[14])

	if ((numSpecialCommentLinesVar >300) || (numSpecialCommentLinesVar <0) )
		close refNum
		Abort "Had trouble reading in the number of special comment lines -  Aborting from Load_LoadHeader1001"
	endif
		
	if (numSpecialCommentLinesVar>0)
		str=""
		for(i=0;i< numSpecialCommentLinesVar; i+=1)
			FReadLine refNum, lineStr
			str+=lineStr
			numLinesHaveBeenRead+=1
		endfor
		ICARTTHeaderValue[15] = str	//	SpecialCommentLines	
	else
		ICARTTHeaderValue[15] = "N/A"	//	SpecialCommentLines		
	endif
	
	FReadLine refNum, lineStr
	ICARTTHeaderValue[16] = lineStr		//	NumNormalCommentLines	
	numLinesHaveBeenRead+=1

	numNormalCommentLinesVar = str2num(ICARTTHeaderValue[16])
	
	if ((numNormalCommentLinesVar >400) || (numVariables <0) )
		close refNum
		Abort "Had trouble reading in the number of normal comment lines. - Aborting from Load_LoadHeader1001"
	endif

//	Dump all the Normal Comments into this placeholder.  Useful if things go awry later.
	FStatus refNum
	i = V_filePos
	str = ""
	for(y=0;y< numNormalCommentLinesVar;y+=1)
		FReadLine refNum, lineStr
		str+=lineStr + "\r"
		numLinesHaveBeenRead+=1
	endfor

	ICARTTHeaderValue[17] = str	//	NormalCommentLines  we'll put the whole thing in here later.	
	FSetPos refNum, i		// set the file position back to the start of the normal comment lines 
	
	FReadLine refNum, lineStr
	ICARTTHeaderValue[18] = lineStr		//	PIContactInfo	
	FReadLine refNum, lineStr
	ICARTTHeaderValue[19] = lineStr		//	PlatformInfo		
	FReadLine refNum, lineStr
	ICARTTHeaderValue[20] = lineStr		//	LocationInfo
	FReadLine refNum, lineStr	
	ICARTTHeaderValue[21] = lineStr		//	AssociatedFileNames

	FReadLine refNum, lineStr	
	ICARTTHeaderValue[22] = lineStr		//	InstrumentInfo	
	FReadLine refNum, lineStr
	ICARTTHeaderValue[23] = lineStr		//	DataInfo
	FReadLine refNum, lineStr
	ICARTTHeaderValue[24] = lineStr		//	UncertaintyInfo		

	FReadLine refNum, lineStr	
	ICARTTHeaderValue[25] = lineStr		//	ULODFlag	
	FReadLine refNum, lineStr
	ICARTTHeaderValue[26] = lineStr		//	ULODValue	

	FReadLine refNum, lineStr	
	ICARTTHeaderValue[27] = lineStr		//	LLODFlag	
	FReadLine refNum, lineStr
	ICARTTHeaderValue[28] = lineStr		//	LLODValue	

	FReadLine refNum, lineStr	
	ICARTTHeaderValue[29] = lineStr		//	DMContactInfo	
	FReadLine refNum, lineStr	
	ICARTTHeaderValue[30] = lineStr		//	ProjectInfo	

	FReadLine refNum, lineStr
	ICARTTHeaderValue[31] = lineStr		//	StipulationsOnUse	

	numLinesHaveBeenRead+=14
	
	str = ""
	i=0
	do
		FReadLine refNum, lineStr
		numLinesHaveBeenRead+=1
		y = cmpstr(lowerstr(lineStr[0,7]), lowerstr("REVISION"))
		if (  (y!=0) && (strlen(lineStr)>0) )
			str += lineStr
		endif
		i+=1
	while( (y!=0) && (strlen(lineStr)>0) &&(i<200) )
	if (i==200)
		Abort "There seems to be something wrong with the header file near the revision comments section - Aborting from Load_LoadHeader1001"
	endif
	
	ICARTTHeaderValue[32] =  str	//	OtherComments	
	
	ICARTTHeaderValue[33] = lineStr	//	RevisionNumberStr	

	str = ""
	do
		FReadLine refNum, lineStr
		str += lineStr
	while(  (strlen(lineStr)>0) && (numLinesHaveBeenRead< NumHeaderLinesVar) )	
	ICARTTHeaderValue[34] = str	//	RevisionStr	
	
	Close refNum
		
End

// *************************************************
// Does cleaning and sanity checking from previous Load_LoadHeader1001 call.

Function Load_IngestHeader1001()
				
	WAVE/t ICARTTHeaderParam = root:ICARTTFileFormat:ICARTTHeaderParam
	WAVE/t ICARTTHeaderValue = root:ICARTTFileFormat:ICARTTHeaderValue
	
	SVAR FileNameStr =root:ICARTTFileFormat:FileNameStr
	SVAR FileNameStr4Macs =root:ICARTTFileFormat:FileNameStr4Macs
 	
	NVAR NumHeaderLinesVar = root:ICARTTFileFormat:NumHeaderLinesVar
	SVAR FileFormatTypeStr =root:ICARTTFileFormat:FileFormatTypeStr

	SVAR PINameStr = root:ICARTTFileFormat:PINameStr
	SVAR PIAffiliationStr = root:ICARTTFileFormat:PIAffiliationStr
	SVAR DataSourceDescriptionStr = root:ICARTTFileFormat:DataSourceDescriptionStr
	SVAR MissionNameStr = root:ICARTTFileFormat:MissionNameStr

	NVAR FileVolumeNumberVar = root:ICARTTFileFormat:FileVolumeNumberVar
	NVAR TotalVolumeNumberVar = root:ICARTTFileFormat:TotalVolumeNumberVar

	SVAR UTCDataStartStr= root:ICARTTFileFormat:UTCDataStartStr
	NVAR UTCDataStartVar = root:ICARTTFileFormat:UTCDataStartVar
	SVAR UTCRevisionDateStr  = root:ICARTTFileFormat:UTCRevisionDateStr

	NVAR DataIntervalVar = root:ICARTTFileFormat:DataIntervalVar

	SVAR IndependentVariableNameStr = root:ICARTTFileFormat:IndependentVariableNameStr
	NVAR NumVariablesVar = root:ICARTTFileFormat:NumVariablesVar

	NVAR ScaleFactorVar  = root:ICARTTFileFormat:ScaleFactorVar
	NVAR MissingDataIndicatorVar = root:ICARTTFileFormat:MissingDataIndicatorVar

	SVAR VariableNamesList = root:ICARTTFileFormat:VariableNamesList

	NVAR NumSpecialCommentLinesVar = root:ICARTTFileFormat:NumSpecialCommentLinesVar
	SVAR SpecialCommentsStr = root:ICARTTFileFormat:SpecialCommentsStr

	NVAR NumNormalCommentLinesVar = root:ICARTTFileFormat:NumNormalCommentLinesVar
	SVAR NormalCommentsStr = root:ICARTTFileFormat:NormalCommentsStr

	SVAR PIContactInfoStr = root:ICARTTFileFormat:PIContactInfoStr
	SVAR PlatformSiteInfoStr  = root:ICARTTFileFormat:PlatformSiteInfoStr
	SVAR LocationInfoStr  = root:ICARTTFileFormat:LocationInfoStr

	SVAR AssociatedFileNamesStr  = root:ICARTTFileFormat:AssociatedFileNamesStr
	SVAR InstrumentInfoStr  = root:ICARTTFileFormat:InstrumentInfoStr
	SVAR DataInfoStr  = root:ICARTTFileFormat:DataInfoStr
	SVAR UncertaintyInfoStr  = root:ICARTTFileFormat:UncertaintyInfoStr

	NVAR ULODFlagVar = root:ICARTTFileFormat:ULODFlagVar
	SVAR ULODValueStr  = root:ICARTTFileFormat:ULODValueStr
	NVAR LLODFlagVar = root:ICARTTFileFormat:LLODFlagVar
	SVAR LLODValueStr  = root:ICARTTFileFormat:LLODValueStr

	SVAR DataManagerContactInfoStr  = root:ICARTTFileFormat:DataManagerContactInfoStr
	SVAR ProjectInfoStr  = root:ICARTTFileFormat:ProjectInfoStr

	SVAR RevisionNumberStr  = root:ICARTTFileFormat:RevisionNumberStr
	SVAR RevisionStr  = root:ICARTTFileFormat:RevisionStr
	
	SVAR StipulationsOnUseStr = root:ICARTTFileFormat:StipulationsOnUseStr
	SVAR OtherCommentsStr = root:ICARTTFileFormat:OtherCommentsStr

	string str

	str = ICARTTHeaderValue[1]
		
	NumHeaderLinesVar = str2num(TeaseOutNumbers(str, 0)	)
	FileFormatTypeStr = TeaseOutNumbers(str, strlen(TeaseOutNumbers(str, 0)))
	FileFormatTypeStr = RemoveTrailingCRorLF(FileFormatTypeStr)
	
	PINameStr = RemoveTrailingCRorLF(  ICARTTHeaderValue[2])
	PIAffiliationStr = RemoveTrailingCRorLF(   ICARTTHeaderValue[3]  )

	DataSourceDescriptionStr = RemoveTrailingCRorLF(  ICARTTHeaderValue[4]  )
	MissionNameStr = RemoveTrailingCRorLF(  ICARTTHeaderValue[5] )

	str = ICARTTHeaderValue[6]

	FileVolumeNumberVar = str2num(TeaseOutNumbers(str, 0)	)
	TotalVolumeNumberVar = str2num(TeaseOutNumbers(str, strlen(TeaseOutNumbers(str, 0))))

	str = ICARTTHeaderValue[7]				

	UTCDataStartStr = Icartt_TeaseOutDate(str, 0)
	UTCDataStartVar = yyyymmddhhmmss2time(UTCDataStartStr[0,3]+UTCDataStartStr[5,6]+UTCDataStartStr[8,9]+"000000")

	variable yearRevPosition	//position in str where date of revision year starts
	yearRevPosition = GetOffsetToRevYear(str)	//returns start of rev string.
	UTCRevisionDateStr =  Icartt_TeaseOutDate(str, yearRevPosition)  // new method to get the revision date string. Replaces line below

//	Line below replaced with line above.The old one has a bug, and also can't deal with data from 1999.
//	UTCRevisionDateStr =  Icartt_TeaseOutDate(str,strsearch(str,  "20", 8))  // cheap but effective; look for space20 after at least 8 chars (4 year,space, num, space, num) 

	DataIntervalVar = str2num(ICARTTHeaderValue[8])
	
	IndependentVariableNameStr = ICARTTHeaderValue[9]

	NumVariablesVar = str2num(ICARTTHeaderValue[10])

	str = ICARTTHeaderValue[11]
	
	ScaleFactorVar = Str2Num(TeaseOutNumbers(str, 0))		// assumes missing value is same for all variables

	str = ICARTTHeaderValue[12]

	MissingDataIndicatorVar = Str2Num(TeaseOutNumbers(str, 0))		// assumes missing value is same for all variables

	VariableNamesList =  StrSubstitute(" ", RemoveCarriageReturns(ICARTTHeaderValue[13]),"") 	//	VariableNames	

	NumSpecialCommentLinesVar = str2num(ICARTTHeaderValue[14])
	
	SpecialCommentsStr = ICARTTHeaderValue[15]
	
	NumNormalCommentLinesVar = Str2Num(ICARTTHeaderValue[16])

	NormalCommentsStr = ICARTTHeaderValue[17] 	
	
	PIContactInfoStr=  RemoveTrailingCRorLF( RemoveTheseFirstChar(ICARTTHeaderValue[18] , "PI_CONTACT_INFO:  ")  )
	PlatformSiteInfoStr = RemoveTrailingCRorLF( RemoveTheseFirstChar(ICARTTHeaderValue[19] , "PLATFORM:  ") )
	LocationInfoStr = RemoveTrailingCRorLF( RemoveTheseFirstChar(ICARTTHeaderValue[20] , "LOCATION:  ") )
	AssociatedFileNamesStr = RemoveTrailingCRorLF( RemoveTheseFirstChar(ICARTTHeaderValue[21] , "ASSOCIATED_DATA:  ") )
	InstrumentInfoStr = RemoveTrailingCRorLF(RemoveTheseFirstChar(ICARTTHeaderValue[22] , "INSTRUMENT_INFO:  ") )
	DataInfoStr = RemoveTrailingCRorLF (RemoveTheseFirstChar(ICARTTHeaderValue[23] , "DATA_INFO:  ") )
	UncertaintyInfoStr = RemoveTrailingCRorLF (RemoveTheseFirstChar(ICARTTHeaderValue[24] , "UNCERTAINTY:  ") )
	ULODFlagVar = str2num(RemoveTrailingCRorLF (RemoveTheseFirstChar(ICARTTHeaderValue[25] , "ULOD_FLAG:  ") ))
	ULODValueStr = RemoveTrailingCRorLF (RemoveTheseFirstChar(ICARTTHeaderValue[26] , "ULOD_VALUE:  ") )
	LLODFlagVar=str2num(RemoveTrailingCRorLF (RemoveTheseFirstChar(ICARTTHeaderValue[27] , "LLOD_FLAG:  ") )  )
	LLODValueStr = RemoveTrailingCRorLF (RemoveTheseFirstChar(ICARTTHeaderValue[28] , "LLOD_VALUE:  ") )
	DataManagerContactInfoStr = RemoveTrailingCRorLF (RemoveTheseFirstChar(ICARTTHeaderValue[29] , "DM_CONTACT_INFO:  ") )
	ProjectInfoStr = RemoveTrailingCRorLF (RemoveTheseFirstChar(ICARTTHeaderValue[30], "PROJECT_INFO:  ") )
	StipulationsOnUseStr= RemoveTrailingCRorLF (RemoveTheseFirstChar(ICARTTHeaderValue[31], "STIPULATIONS_ON_USE:  ") )
	OtherCommentsStr= RemoveTrailingCRorLF (RemoveTheseFirstChar(ICARTTHeaderValue[32], "OTHER_COMMENTS:  ") )
	RevisionNumberStr = RemoveTrailingCRorLF (RemoveTheseFirstChar(ICARTTHeaderValue[33], "REVISION:  ") )

	RevisionStr =RemoveTrailingCRorLF ( ICARTTHeaderValue[34] )
			
End


// *************************************************
//  Loads text data from the data file into waves.  Prompts the user if waves will be overwritten.
// Creates and sets the global string VariableNamesList

Function Load_LoadData1001(fileName)
	String fileName

	NVAR numHeaderLinesVar = root:ICARTTFileFormat:numHeaderLinesVar
	SVAR VariableNamesList= root:ICARTTFileFormat:VariableNamesList
		
	LoadWave/A/P=IcarttFilePath/G/W/K=0/V={" "," $",0,1}/L={numHeaderLinesVar-1,numHeaderLinesVar,0,0,0} fileName		// loads as single precision
	VariableNamesList = S_waveNames
		
  	IcarttButton_UpdateWavesList("buttonUpdateWaveList")

End


// *************************************************
// Replaces the missing values for all dependent variables with nans.
// First does CheckNumVariablesICARTT1001 run so that the numVariablesVar is correct
// To all time waves it changes dimension to double, adds UTCDataStartVar value to it, and sets wave scaling to date&time.

Function Load_ChangeMissingDoDate1001()

	NVAR MissingDataIndicatorVar = root:ICARTTFileFormat:MissingDataIndicatorVar
	NVAR UTCDataStartVar = root:ICARTTFileFormat:UTCDataStartVar
	SVAR VariableNamesList= root:ICARTTFileFormat:VariableNamesList		// actually loaded wave names - not what was indicated in the header
	SVAR IndependentVariableNameStr= root:ICARTTFileFormat:IndependentVariableNameStr	// actually loaded wave names - not what was indicated in the header
	
	variable i, n
	string nameStr
			
	n = ItemsInList(VariableNamesList)
	
	for (i=0; i<n; i+=1)

		nameStr = StringFromList(i, VariableNamesList, ";")

		wave w = $nameStr

		if (WaveExists(w) ==0)
			DoAlert 1,  "The wave "+namestr+" does not exist.  Continue with processing?"
			if (V_flag == 2)
				Abort "The function was canceled at user request - Aborting from Load_ChangeMissingDoDate1001"
			endif
		else
			w = SelectNumber (round(w[p]) == MissingDataIndicatorVar, w[p], nan)
			if (Icartt_IsATimeString(namestr) )
				Redimension/D w
				w+=UTCDataStartVar
				SetScale d 0,0,"dat", w 
			endif			
		endif		// the wave exists
		
	endfor

End


// *************************************************
// Creates an icartt header file.

Function Create_CreateHeader()
			
	SVAR FileNameStr = root:ICARTTFileFormat:FileNameStr
	SVAR FileNameStr4Macs = root:ICARTTFileFormat:FileNameStr4Macs
	
	NVAR UTCDataStartVar = root:ICARTTFileFormat:UTCDataStartVar

	NVAR NumHeaderLinesVar = root:ICARTTFileFormat:NumHeaderLinesVar
	SVAR FileFormatTypeStr = root:ICARTTFileFormat:FileFormatTypeStr
	SVAR PINameStr = root:ICARTTFileFormat:PINameStr
	SVAR PIAffiliationStr  = root:ICARTTFileFormat:PIAffiliationStr
	SVAR DataSourceDescriptionStr  = root:ICARTTFileFormat:DataSourceDescriptionStr
	SVAR MissionNameStr = root:ICARTTFileFormat:MissionNameStr
	NVAR FileVolumeNumberVar = root:ICARTTFileFormat:FileVolumeNumberVar
	NVAR TotalVolumeNumberVar = root:ICARTTFileFormat:TotalVolumeNumberVar
	SVAR UTCDataStartStr = root:ICARTTFileFormat:UTCDataStartStr
	SVAR UTCRevisionDateStr = root:ICARTTFileFormat:UTCRevisionDateStr
	NVAR DataIntervalVar = root:ICARTTFileFormat:DataIntervalVar
	SVAR IndependentVariableNameStr = root:ICARTTFileFormat:IndependentVariableNameStr
	NVAR NumVariablesVar = root:ICARTTFileFormat:NumVariablesVar
	NVAR ScaleFactorVar = root:ICARTTFileFormat:ScaleFactorVar
	NVAR MissingDataIndicatorVar = root:ICARTTFileFormat:MissingDataIndicatorVar
	SVAR VariableNamesList = root:ICARTTFileFormat:VariableNamesList

	NVAR NumSpecialCommentLinesVar = root:ICARTTFileFormat:NumSpecialCommentLinesVar
	SVAR SpecialCommentsStr = root:ICARTTFileFormat:SpecialCommentsStr

	NVAR NumNormalCommentLinesVar = root:ICARTTFileFormat:NumNormalCommentLinesVar
	SVAR NormalCommentsStr = root:ICARTTFileFormat:NormalCommentsStr

	SVAR PIContactInfoStr = root:ICARTTFileFormat:PIContactInfoStr
	SVAR PlatformSiteInfoStr = root:ICARTTFileFormat:PlatformSiteInfoStr
	SVAR LocationInfoStr = root:ICARTTFileFormat:LocationInfoStr
	SVAR AssociatedFileNamesStr = root:ICARTTFileFormat:AssociatedFileNamesStr
	SVAR InstrumentInfoStr = root:ICARTTFileFormat:InstrumentInfoStr
	SVAR DataInfoStr = root:ICARTTFileFormat:DataInfoStr
	SVAR UncertaintyInfoStr = root:ICARTTFileFormat:UncertaintyInfoStr
	NVAR ULODFlagVar = root:ICARTTFileFormat:ULODFlagVar
	SVAR ULODValueStr = root:ICARTTFileFormat:ULODValueStr
	NVAR LLODFlagVar= root:ICARTTFileFormat:LLODFlagVar
	SVAR LLODValueStr	= root:ICARTTFileFormat:LLODValueStr
	SVAR DataManagerContactInfoStr= root:ICARTTFileFormat:DataManagerContactInfoStr
	SVAR ProjectInfoStr= root:ICARTTFileFormat:ProjectInfoStr
	SVAR StipulationsOnUseStr = root:ICARTTFileFormat:StipulationsOnUseStr
	NVAR NumOtherCommentsVar= root:ICARTTFileFormat:NumOtherCommentsVar
	SVAR OtherCommentsStr = root:ICARTTFileFormat:OtherCommentsStr
	SVAR RevisionNumberStr = root:ICARTTFileFormat:RevisionNumberStr
	SVAR RevisionStr= root:ICARTTFileFormat:RevisionStr

	WAVE/T WaveListTextWave	= root:ICARTTFileFormat:WaveListTextWave
	WAVE/T WaveListTextWaveMatrix= root:ICARTTFileFormat:WaveListTextWaveMatrix	// for ListBox
	WAVE SelWaveMatrix= root:ICARTTFileFormat:SelWaveMatrix 	// for ListBox
		
	SVAR IcarttPathStr = root:ICARTTFileFormat:Panel:IcarttPathStr

	variable fileNum, i
	string str, cmd
	
	Create_CheckSanitizeHead1001()

	Icartt_SetIcartFilePath()

	Create_CheckFileNameLength4Mac()
	
	Open/P=ICARTTFilePath fileNum as FileNameStr			// Will overwrite if exists
	
	// \r\n is carriage return line feed
	
	Fprintf fileNum, "%s %s\r\n", num2str(NumHeaderLinesVar), FileFormatTypeStr
	Fprintf fileNum, "%s\r\n", PINameStr
	Fprintf fileNum, "%s\r\n",  PIAffiliationStr 
	Fprintf fileNum, "%s\r\n", DataSourceDescriptionStr 
	Fprintf fileNum, "%s\r\n", MissionNameStr
	Fprintf fileNum, "%s %s\r\n", num2str(FileVolumeNumberVar), num2str(TotalVolumeNumberVar)
	Fprintf fileNum, "%s %s\r\n", UTCDataStartStr, UTCRevisionDateStr

	Fprintf fileNum, "%s\r\n", num2str(DataIntervalVar)
	Fprintf fileNum, "%s\r\n", IndependentVariableNameStr
	Fprintf fileNum, "%s\r\n", num2str(NumVariablesVar)
	
	for (i=0; i<NumVariablesVar; i+=1)
		Fprintf fileNum, "%s ", num2str(ScaleFactorVar)
	endfor
	Fprintf fileNum, "\r\n"
	
	for (i=0; i<NumVariablesVar; i+=1)
		Fprintf fileNum, "%s ", num2str(MissingDataIndicatorVar)
	endfor
	Fprintf fileNum, "\r\n"

	for (i=0; i<NumVariablesVar; i+=1)
		Fprintf fileNum, "%s\r\n",  StringFromList(i, VariableNamesList, ";")
	endfor

	Fprintf fileNum, "%s\r\n", num2str(NumSpecialCommentLinesVar)

	if (NumSpecialCommentLinesVar>0)
		FprintfLongerThan1000Chars(fileNum, SpecialCommentsStr)
	endif	

	Fprintf fileNum, "%s\r\n", num2str(NumNormalCommentLinesVar)

	Fprintf fileNum, "%s\r\n", "PI_CONTACT_INFO: "+PIContactInfoStr
	Fprintf fileNum, "%s\r\n", "PLATFORM: "+PlatformSiteInfoStr 
	Fprintf fileNum, "%s\r\n", "LOCATION: "+LocationInfoStr
	Fprintf fileNum, "%s\r\n", "ASSOCIATED_DATA: "+AssociatedFileNamesStr
	Fprintf fileNum, "%s\r\n", "INSTRUMENT_INFO: "+InstrumentInfoStr
	Fprintf fileNum, "%s\r\n", "DATA_INFO: "+DataInfoStr
	Fprintf fileNum, "%s\r\n", "UNCERTAINTY: "+UncertaintyInfoStr
	Fprintf fileNum, "%s\r\n", "ULOD_FLAG: "+num2str(ULODFlagVar)
	Fprintf fileNum, "%s\r\n", "ULOD_VALUE: "+ULODValueStr
	Fprintf fileNum, "%s\r\n", "LLOD_FLAG: "+num2str(LLODFlagVar)
	Fprintf fileNum, "%s\r\n", "LLOD_VALUE: "+LLODValueStr
	Fprintf fileNum, "%s\r\n", "DM_CONTACT_INFO: "+DataManagerContactInfoStr
	Fprintf fileNum, "%s\r\n", "PROJECT_INFO: "+ProjectInfoStr
	Fprintf fileNum, "%s\r\n", "STIPULATIONS_ON_USE: "+StipulationsOnUseStr
	Fprintf fileNum, "%s", "OTHER_COMMENTS: "
		FprintfLongerThan1000Chars(fileNum, OtherCommentsStr)		

	Fprintf fileNum, "%s\r\n", "REVISION: "+RevisionNumberStr
		FPrintFLongerThan1000Chars(fileNum, RevisionStr)		
	Close fileNum
	
	DoWindow/F Icartt_Panel
	
End


// *************************************************
// Checks to see if igor path IcarttFilePath exists and prompt user if it doesn't.

Function Icartt_SetIcartFilePath()

	SVAR IcarttPathStr = root:ICARTTFileFormat:Panel:IcarttPathStr

	PathInfo ICARTTFilePath		// creates or overwrites the path where the file is located.			
	if (V_flag==0)
		NewPath/M="Please select a path for the icartt files" ICARTTFilePath
		if (V_flag!=0)
			Abort "You must select a folder for the igor Icartt file path before proceeding - Aborting from Icartt_CheckFileNameStr"
		endif
	endif
	
	IcarttPathStr = ConvertPathStr("IcarttFilePath")

End


// *************************************************
// Aborts the creation of an icartt file if the name >32 chars AND user is on a Mac plaform.

Function Create_CheckFileNameLength4Mac()

	SVAR FileNameStr = root:ICARTTFileFormat:FileNameStr

	if ((strlen(FileNameStr)>=32)&& (cmpstr("Macintosh", IgorInfo(2))==0))
		Abort "Igor for the Mac does not support creating files with names > 32 chars.  Please change the filename.  - Aborting from CheckCreateFileNamesLength4Macs"
	endif

End


// *************************************************
// Creates or appends data columns to icartt file.

Function Create_AppendData2Header()
		
	NVAR DataIntervalVar = root:ICARTTFileFormat:DataIntervalVar
	SVAR VariableNamesList = root:ICARTTFileFormat:VariableNamesList	
	NVAR UTCDataStartVar = root:ICARTTFileFormat:UTCDataStartVar
	SVAR IndependentVariableNameStr= root:ICARTTFileFormat:IndependentVariableNameStr
	NVAR NumVariablesVar= root:ICARTTFileFormat:NumVariablesVar
	NVAR MissingDataIndicatorVar= root:ICARTTFileFormat:MissingDataIndicatorVar
	NVAR FormatAsInTopTableVar= root:ICARTTFileFormat:FormatAsInTopTableVar

	WAVE/T WaveListTextWave	= root:ICARTTFileFormat:WaveListTextWave
	WAVE/T WaveListTextWaveMatrix = root:ICARTTFileFormat:WaveListTextWaveMatrix	// for ListBox
	WAVE SelWaveMatrix = root:ICARTTFileFormat:SelWaveMatrix	// for ListBox
		
	SVAR MatrixColNameList = root:ICARTTFileFormat:MatrixColNameList
	NVAR IsMatrixData = root:ICARTTFileFormat:IsMatrixData
	
	SVAR FileNameStr = root:ICARTTFileFormat:FileNameStr
	SVAR FileNameStr4Macs = root:ICARTTFileFormat:FileNameStr4Macs
	
	string fileName 
	string formatVarNameStr
	variable fileNum, i
	string thisCmd
	string colFormatStr1, colFormatStr2, tableName
		
	Create_CheckFileNameLength4Mac()

	Icartt_SetIcartFilePath()
		
	VariableNamesList =  Create_CreateListVarStr()
	NumVariablesVar = ItemsInList(VariableNamesList, ";")
	
	formatVarNameStr =  Create_CreateFormatVarStr()
	
	WAVE twave = $IndependentVariableNameStr
	Duplicate/o twave tw
	tw	= twave - UTCDataStartVar
		
	Create_DupChangeNansSeltVar()

	Open/A/P=ICARTTFilePath fileNum as FileNameStr
	
	Fprintf  fileNum, "%s" 	IndependentVariableNameStr+ " "
 	for (i = 0; i<=ItemsInList(VariableNamesList, ";"); i+=1 )
		Fprintf  fileNum, "%s" StringFromList(i, VariableNamesList, ";")+ " "
	endfor
	Fprintf  fileNum, "\r\n" 

	// get ready to print column data
	thisCmd = "wfprintf "+num2str(fileNum)+", \""+formatVarNameStr+"\\r\\n\" tw" 
 	
 	for (i = 1; i<=NumVariablesVar; i+=1 )
		thisCmd+=", X"+num2str2char(i)
	endfor

	if (FormatAsInTopTableVar ==0)
		if (strlen(ThisCmd)< 400)		
			Execute thisCmd
			Close fileNum
		else 
			Close fileNum
			Print thisCmd, strlen(thisCmd)
			Abort "Cannot use the execute command with a string so long.   Retry using tables - Aborting from Create_AppendData2Header"
		endif		
	else
		Close fileNum
		tableName = Create_CheckFormatAsInTable()
		print tableName
		
		DoWindow/K MyIcarttTable
		Edit/N=MyIcarttTable tw
		for (i = 1; i<=NumVariablesVar; i+=1 )
			AppendtoTable $("X"+num2str2char(i))
			colFormatStr1 = TableInfo(TableName,i)
			colFormatStr2 = StringByKey("FORMAT", colFormatStr1) 
			if (cmpstr(colFormatStr2, "8") == 0 )  // we are dealing with a timewave
				ModifyTable format($("X"+num2str2char(i)))=3
				ModifyTable digits($("X"+num2str2char(i)))=1
			else
				ModifyTable format($("X"+num2str2char(i)))=str2num(colFormatStr2)
				colFormatStr2 = StringByKey("DIGITS", colFormatStr1) 
				ModifyTable digits($("X"+num2str2char(i)))=str2num(colFormatStr2)
			endif
		endfor
		ModifyTable format(tw)=3, digits(tw)=1		// for timewave
		
		SaveTableCopy/M="\r\n"/A=2/N=0/o/S=0/T=3/P=ICARTTFilePath as FileNameStr		// use tabs as separators, do not write column names.
		DoWindow/K MyIcarttTable
		
	endif

	KillwavesInList("tw")
	KillwavesInList(WaveList("X*", ";", "" ))
	if (IsMatrixData)
		DoWindow/K IcarttMatrixDataTable
		KillwavesInList(MatrixColNameList)
	endif
	
  	IcarttButton_UpdateWavesList("buttonUpdateWaveList")
  
End


// *************************************************
// Subtracts UTCDataStartVar value from temporary time waves.

Function Create_AdjustIfTimewave(MyTempWave, str)
	Wave MyTempWave
	String str

	WAVE/T WaveListTextWave	= root:ICARTTFileFormat:WaveListTextWave
	WAVE/T WaveListTextWaveMatrix = root:ICARTTFileFormat:WaveListTextWaveMatrix	// for ListBox

	NVAR UTCDataStartVar = root:ICARTTFileFormat:UTCDataStartVar

	string formatStr
	variable n
	
	n = TextInTextwave(str, WaveListTextWave)		// assumes WaveListTextWave and WaveListTextWaveMatrix ordered the same.
	
	if (n>=0)
		formatStr = WaveListTextWaveMatrix[n][0]
		if (cmpstr(formatStr, "t") ==0)
			MyTempWave -=UTCDataStartVar
		endif
	endif

End


// *************************************************
// Duplicates waves to be put in icartt file, names them X_01, etc. and fills in the nans with missing values.

Function Create_DupChangeNansSeltVar()

	NVAR NumVariablesVar = root:ICARTTFileFormat:NumVariablesVar	
	SVAR VariableNamesList = root:ICARTTFileFormat:VariableNamesList	
	NVAR MissingDataIndicatorVar= root:ICARTTFileFormat:MissingDataIndicatorVar
		
	WAVE/T WaveListTextWave	= root:ICARTTFileFormat:WaveListTextWave
	WAVE/T WaveListTextWaveMatrix = root:ICARTTFileFormat:WaveListTextWaveMatrix	// for ListBox
	WAVE SelWaveMatrix = root:ICARTTFileFormat:SelWaveMatrix	// for ListBox

	string str
	variable i

	VariableNamesList =  Create_CreateListVarStr()
	NumVariablesVar = ItemsInList(VariableNamesList, ";")
	 
	 if (NumVariablesVar==0)
		Abort "Please select some dependent parameters. - Aborting from Create_DupChangeNansSeltVar"
	endif
	
	for(i=NumVariablesVar; i>0;i-=1)
		str = StringFromList(i-1, VariableNamesList, ";")
		wave w = $str
		if (WaveExists(w)==0)
			print "// This wave doesn't exist "+NameofWave(w) + "*"+str+"*"
		endif
		Duplicate/o w $("X"+num2str2char(i) )
		ChangeNans($("X"+num2str2char(i) ),MissingDataIndicatorVar )
		Create_AdjustIfTimewave($("X"+num2str2char(i) ), str)
	endfor

End


// *************************************************
//  Returns an important string which will have data column formatting info in it.

Function/s Create_CreateFormatVarStr()

	WAVE/T WaveListTextWaveMatrix = root:ICARTTFileFormat:WaveListTextWaveMatrix	// for ListBox
	WAVE SelWaveMatrix = root:ICARTTFileFormat:SelWaveMatrix	// for ListBox

	NVAR ExponFormatVar = root:ICARTTFileFormat:ExponFormatVar	

	variable i, n, numRows = DimSize(SelWaveMatrix, 0)
	string returnStr, str
	
	returnStr = "%1.1f "
	for (i=0; i<numRows; i+=1)
		if (SelWaveMatrix[i][0]==3)	// it is selected
			str = WaveListTextWaveMatrix[i][0]
			if (numtype (str2num(str)) == 0)// we have a number
				returnStr+="%0."+num2str((str2num(str))) + "f "
			elseif(cmpstr(str, "e")==0)
				returnStr+="%1."+num2str(ExponFormatVar)+"e "
			elseif(cmpstr(str, "t")==0)
				returnStr+="%1.1f "					
			else
				Abort "You did not enter an appropriate value in the first column - Aborting from Create_CreateFormatVarStr"
			endif
		endif	
 	endfor
	returnStr = returnStr[0, strlen(returnStr)-2]		// we always append a space
	
	return returnStr
	
End


// *************************************************
// Returns a string which has all the waves names which were selected in the panel.

Function/s Create_CreateListVarStr()

	WAVE/T WaveListTextWaveMatrix = root:ICARTTFileFormat:WaveListTextWaveMatrix	// for ListBox
	WAVE SelWaveMatrix = root:ICARTTFileFormat:SelWaveMatrix	// for ListBox

	variable i=0, n, numRows = DimSize(SelWaveMatrix, 0)
	string returnStr, str
	
	returnStr = ""
	for (i=0; i<numRows; i+=1)
		if (SelWaveMatrix[i][0]==3)	// it is selected
			str = WaveListTextWaveMatrix[i][2]			
			returnStr+=str + ";"	
		endif	
 	endfor
	returnStr = returnstr[0, strlen(returnstr)-2]			// append a space
	
	return returnStr
	
End
	
	
// *************************************************
// Performs a variety of sanity checks; useful for checking values immediately previous to writing a file.

Function Create_CheckSanitizeHead1001()
	
	SVAR FileNameStr =root:ICARTTFileFormat:FileNameStr
	SVAR FileNameStr4Macs =root:ICARTTFileFormat:FileNameStr4Macs
	
	NVAR NumHeaderLinesVar =root:ICARTTFileFormat:NumHeaderLinesVar

	NVAR UTCDataStartVar =root:ICARTTFileFormat:UTCDataStartVar
	SVAR UTCDataStartStr =root:ICARTTFileFormat:UTCDataStartStr
	SVAR UTCRevisionDateStr  =root:ICARTTFileFormat:UTCRevisionDateStr

	NVAR DataIntervalVar =root:ICARTTFileFormat:DataIntervalVar
	SVAR IndependentVariableNameStr =root:ICARTTFileFormat:IndependentVariableNameStr

	NVAR NumVariablesVar =root:ICARTTFileFormat:NumVariablesVar
	SVAR VariableNamesList =root:ICARTTFileFormat:VariableNamesList

	NVAR NumSpecialCommentLinesVar =root:ICARTTFileFormat:NumSpecialCommentLinesVar
	SVAR SpecialCommentsStr =root:ICARTTFileFormat:SpecialCommentsStr
	NVAR NumNormalCommentLinesVar =root:ICARTTFileFormat:NumNormalCommentLinesVar
	SVAR NormalCommentsStr =root:ICARTTFileFormat:NormalCommentsStr

	NVAR NumOtherCommentsVar =root:ICARTTFileFormat:NumOtherCommentsVar
	SVAR OtherCommentsStr =root:ICARTTFileFormat:OtherCommentsStr

	SVAR RevisionNumberStr =root:ICARTTFileFormat:RevisionNumberStr
	SVAR RevisionStr =root:ICARTTFileFormat:RevisionStr

	WAVE/t WaveListTextWaveMatrix=root:ICARTTFileFormat:WaveListTextWaveMatrix
	WAVE SelWaveMatrix=root:ICARTTFileFormat:SelWaveMatrix

	SVAR PINameStr = root:ICARTTFileFormat:PINameStr
	
	SVAR MergedSuffixStr = root:ICARTTFileFormat:MergedSuffixStr

	variable i,y
	string str, filenNme, oneLineList
	
	oneLineList = "PINameStr;PIAffiliationStr;DataSourceDescriptionStr;MissionNameStr;"
	oneLineList+="PIContactInfoStr;PlatformSiteInfoStr;LocationInfoStr;AssociatedFileNamesStr;"
	oneLineList+="InstrumentInfoStr;DataInfoStr;UncertaintyInfoStr;ULODValueStr;LLODValueStr;ProjectInfoStr;StipulationsOnUseStr"
		
	Icartt_CheckFileNameStr()
	Icartt_CheckHeaderDate()
	Icartt_CheckRevStrSetNumber()
	
// checks if the independent wave is an existing wave
	IndependentVariableNameStr = RemoveDoubleSpaces(RemoveCarriageReturns(IndependentVariableNameStr))
	wave indepwave = $IndependentVariableNameStr
	if (WaveExists(indepwave)==0)
		Abort "The independent wave "+IndependentVariableNameStr+" does not exist. - Aborting from Create_CheckSanitizeHead1001"
	endif

// sets the UTCDataStartVar
	UTCDataStartVar =indepwave[0] - mod(indepwave[0] , 86400)

// checks if the UTCDataStartVar matches the first value in IndependentVariableNameStr
	if( UTCDataStartVar!= yyyymmddhhmmss2time(UTCDataStartStr[0,3]+UTCDataStartStr[5,6]+UTCDataStartStr[8,9]+"000000" ))
		Abort "The timewave and the UTC start date don't seem to match. - Aborting from Create_CheckSanitizeHead1001"
	endif

	UTCRevisionDateStr = RemoveDoubleSpaces(RemoveCarriageReturns(UTCRevisionDateStr))

// checks UTCRevisionDateStr and the UTCDataStartStr
	Icartt_CheckUTCDateStr(UTCRevisionDateStr)		
	Icartt_CheckUTCDateStr(UTCDataStartStr)		
	 	
	str = time2yyyymmddhhmmss(DateTime)
	str = str[0,7]
	if (cmpstr( StrSubstitute(" ",UTCRevisionDateStr,""), str)!=0)
		DoAlert 1,  "The revision date is not today's date.  Click yes to use the existing revision date, no to abort."
		if (V_flag == 2)
			Abort "The function was cancelled at the user's request - Aborting from Create_CheckSanitizeHead1001"
		endif
	endif

//  Checks the number of dependent parameters.
	NumVariablesVar = 0
	VariableNamesList = ""
	for (i=0; i<dimsize(SelWaveMatrix,0); i+=1)
		if (SelWaveMatrix[i][0] ==3)
			NumVariablesVar+=1
			VariableNamesList+=WaveListTextWaveMatrix[i][1]+";"
		endif
	endfor

	if (NumVariablesVar==0)
		Abort "No dependent parameters were selected  - Aborting from Create_CheckSanitizeHead1001"
	endif
	 	
	if (NumVariablesVar>=200)
		Abort "More than 200 dependent parameters were selected - Aborting from Create_CheckSanitizeHead1001 "
	endif

// Checks that the number of points is the same in the dependent parameters and the independent parameter 
	
	if ( AllInListSameLength(IndependentVariableNameStr+";"+VariableNamesList) ==0)
		Abort "The number of points in the wave "+VariableNamesList+ " and the independent parameter are not the same. - Aborting from Create_CheckSanitizeHead1001"
	endif

	Icartt_CheckIndependentVarTime()

	Create_RemoveExtaLinesAndChars(oneLineList)
	
//  Clean up string parameters that can have more than one carriage return
	SpecialCommentsStr = RemoveTrailingCarriageReturns(SpecialCommentsStr)
	if (strlen(SpecialCommentsStr)==0)
		NumSpecialCommentLinesVar=0
	else
		NumSpecialCommentLinesVar = CountCarriageReturns(SpecialCommentsStr)+1
	endif
	
	OtherCommentsStr =  RemoveTrailingCarriageReturns(OtherCommentsStr)	
	if (CountCarriageReturns(OtherCommentsStr)==0)
		NumOtherCommentsVar = 0
	else
		NumOtherCommentsVar = CountCarriageReturns(OtherCommentsStr)+1	
	endif

//  Check the RevisionStr
	RevisionStr = RemoveTrailingCarriageReturns(RevisionStr)
	y = CountCarriageReturns(RevisionStr)

	NumOtherCommentsVar = CountCarriageReturns(OtherCommentsStr)
				
	NumNormalCommentLinesVar = 16+ CountCarriageReturns(RevisionStr)+1+(NumOtherCommentsVar) + 1

	NumHeaderLinesVar =12 + NumVariablesVar+(NumSpecialCommentLinesVar+1)+(NumNormalCommentLinesVar+1)  // The last one is the parameter names

	Icartt_CheckFileNameStr()
	
//  Check the PINameStr
	if (strlen(PINameStr)==0)
		Abort "Please insert a non-empty entry in the PI Name parameter. - Aborting from Create_CheckSanitizeHead1001"
	endif
		
End


// *************************************************
// Loads an icartt file.

Function LoadICARTT1001()

	Icartt_CreateGlobals()
	IcarttPanel_CreateGlobals()
	 	
	SVAR FileNameStr = root:ICARTTFileFormat:FileNameStr
	SVAR FileNameStr4Macs = root:ICARTTFileFormat:FileNameStr4Macs
	
	SVAR FileFormatTypeStr= root:ICARTTFileFormat:FileFormatTypeStr
					
	string fileName
	
	Icartt_SelectFileDialog()	// sets FileNameStr and FileNameStr4macs 
		
	if (cmpstr("Macintosh", IgorInfo(2))==0  )		// Macs can truncate filenames
		FileName = FileNameStr4Macs
	else
		FileName = FileNameStr
	endif
	
	if (strlen(FileName)>0)
								
		Load_GetNumHeaderLinesFileTyp(FileName)
			
		if (cmpstr("1001", FileFormatTypeStr) ==0)

			Load_LoadHeader1001(FileName)
			
			Load_IngestHeader1001()
			
			Load_SetDateFromFilename()
			
			Load_LoadData1001(FileName)
			
			Load_ChangeMissingDoDate1001()
	
		else
			Abort "File was not of the type 1001.  Did not load. - Aborting from LoadICARTT1001"
		endif
	endif
	
End


// *************************************************
// Loads only data columns from an icartt file; does not change globals used for subsequent creation of an icartt header.

Function LoadICARTTDataOnly1001()

	Icartt_CreateGlobals()
	IcarttPanel_CreateGlobals()
	 
	SVAR FileFormatTypeStr = root:ICARTTFileFormat:FileFormatTypeStr

	SVAR FileNameStr4Macs= root:ICARTTFileFormat:FileNameStr4Macs
	SVAR FileNameStr= root:ICARTTFileFormat:FileNameStr

	string pathStr, fileName
		
	Icartt_SelectFileDialog()		// sets FileNameStr4Macs and FileNameStr globals
		
	if (cmpstr("Macintosh", IgorInfo(2))==0  )		// Macs can truncate filenames
		fileName = FileNameStr4Macs
	else
		fileName = FileNameStr
	endif

	if (strlen(fileName)>0)
										
		Load_GetNumHeaderLinesFileTyp(fileName)

		if (cmpstr("1001", FileFormatTypeStr) ==0)			
			Load_LoadData1001(fileName)			
		else
			Abort "File was not of the type 1001.  Did not load - Aborting from Load_Load1001"
		endif

	endif
	
End


// *************************************************
// Sets everything up for subsequent icartt file creation via the icartt panel.

Function CreateICARTT1001()

	Icartt_CreateGlobals()
	IcarttPanel_CreateGlobals()

	SVAR FileNameStr4Macs= root:ICARTTFileFormat:FileNameStr4Macs
	SVAR FileNameStr= root:ICARTTFileFormat:FileNameStr

	string fileName
	
	DoAlert 2, "Click Yes if you would like to load header info from an existing ICARTT file.  \rClick No if you want to use generic default values."
	
	if (V_flag ==1)	// Yes is clicked

		Icartt_SelectFileDialog()	// sets FileNameStr and FileNameStr4macs
			
		if (cmpstr("Macintosh", IgorInfo(2))==0  )		// Macs can truncate filenames
			fileName = FileNameStr4Macs
		else
			fileName = FileNameStr
		endif

		if (strlen(fileName)>0)
			Load_LoadHeader1001(FileName)
			Load_IngestHeader1001()
			IcarttPanel_CreatePanel()
		endif		

	else
		if(V_flag==2) // No is clicked
			 IcarttPanel_CreatePanel()
		endif
	endif

End


// *************************************************
// Removes extraneous characters from many global variables and sets some globals that may have been changed from a previous icartt file load.

Function  Create_RemoveExtaLinesAndChars(strList)
	String strList

	SVAR FileFormatTypeStr = root:ICARTTFileFormat:FileFormatTypeStr

	NVAR FileVolumeNumberVar = root:ICARTTFileFormat:FileVolumeNumberVar
	NVAR TotalVolumeNumberVar = root:ICARTTFileFormat:TotalVolumeNumberVar
	NVAR ScaleFactorVar = root:ICARTTFileFormat:ScaleFactorVar
	NVAR MissingDataIndicatorVar = root:ICARTTFileFormat:MissingDataIndicatorVar

	variable i, n
	
	n = ItemsInList(strList)
	
//  Resets ScaleFactorVar and MissingDataIndicatorVar in the case that there were changed on a load file.
	FileFormatTypeStr = "1001"
	
	FileVolumeNumberVar = 1
	TotalVolumeNumberVar = 1
	ScaleFactorVar = 1
	MissingDataIndicatorVar = -9999

	for(i=0;i<n;i+=1)
		SVAR str = $"root:ICARTTFileFormat:"+StringFromList(i, strlist )
		if (SVAR_Exists(str)==0)
			Abort "Had problems with the global variable "+StringFromList(i, strlist )+" - Aborting from Create_RemoveExtaLinesAndChars"
		else
			str = RemoveDoubleSpaces(RemoveCarriageReturns(str))
		endif
	endfor

End


// *************************************************
// Returns a nicely formated yyyy mm dd string.
// Ex. print TeaseOutDate("2004 7 5 2005 3 7", 8) returns 2005 03 07
// Ex. print TeaseOutDate("2004 7 5 2005 3 7", 0) returns 2004 07 05

Function/S Icartt_TeaseOutDate(str, startPos)
	String str
	Variable startPos

	string returnStr = ""
	string myChar, mmdd
	variable finished = 0
	variable i, n = strlen(str)
	
	i = startPos
	
	do
		myChar = str[i]
		if(FindListItem(myChar, "0;1;2;3;4;5;6;7;8;9")  >=0)
			returnStr +=myChar
		endif
		i+=1 	
	while( (strlen(returnStr)<4) && (i<n) )
	
	mmdd = ""
	do
		myChar = str[i]
		if(FindListItem(myChar, "0;1;2;3;4;5;6;7;8;9")  >=0)
			mmdd +=myChar
		endif 	
		i+=1 	
	while( (strlen(mmdd)==0) || (FindListItem(myChar, "0;1;2;3;4;5;6;7;8;9")  >=0) )

	if (strlen(mmdd)==1)
		returnStr+=" 0"+mmdd
	else
		returnStr+=" "+mmdd
	endif
	
	mmdd = ""
	do
		myChar = str[i]
		if(FindListItem(myChar, "0;1;2;3;4;5;6;7;8;9")  >=0)
			mmdd +=myChar
		endif 	
		i+=1 	
	while( (strlen(mmdd)==0)  ||  (FindListItem(myChar, "0;1;2;3;4;5;6;7;8;9")  >=0) )

	if (strlen(mmdd)==1)
		returnStr+=" 0"+mmdd
	else
		returnStr+=" "+mmdd
	endif

	Return returnStr

End

// *************************************************
// Sets parameters in panel to reflect formatting in top table.

Function/S Create_CheckFormatAsInTable()

	SVAR IndependentVariableNameStr = root:ICARTTFileFormat:IndependentVariableNameStr
	NVAR FormatAsInTopTableVar = root:ICARTTFileFormat:FormatAsInTopTableVar
		
	string tableName, myWaveStr

	IcarttButton_UpdateWavesList("button_UpdateWaveList")
		
	if (FormatAsInTopTableVar)
		tableName = WinName(0,2)
		if (strlen(tableName)==0)
			Abort "there is no top table - Aborting from CheckFormatAsInTable"
		endif
				
		myWaveStr = StringByKey("COLUMNNAME", TableInfo(tableName, 0))
		myWaveStr = myWaveStr[0, strsearch(myWaveStr, ".", 0) -1] 		// strip out the ".d" suffix

		if (cmpstr(myWaveStr, IndependentVariableNameStr)!=0)
			Abort "The first column in the table must match the parameter selected in step 1 - Aborting from CheckFormatAsInTable"
		endif
			
		return tableName
	else
		return ""	
	endif
	
End


// *************************************************
// Sets the order of parameters to reflect those in the top table.

Function Create_ReOrderWavesFromTable()

	SVAR MyTempStr = root:ICARTTFileFormat:MyTempStr
	SVAR MyWaveListStr = root:ICARTTFileFormat:MyWaveListStr
	SVAR IndependentVariableNameStr = root:ICARTTFileFormat:IndependentVariableNameStr
	
	NVAR FormatAsInTopTableVar = root:ICARTTFileFormat:FormatAsInTopTableVar
	
	WAVE/T TempTextWave = root:ICARTTFileFormat:TempTextWave

	string tableName, myList	
	variable i, n
	
	if (FormatAsInTopTableVar)
		tableName = Create_CheckFormatAsInTable()
			
		myList = ""
		i=0
		
		do
		 	Wave/Z w = WaveRefIndexed(tableName, i, 1)
		       if (!WaveExists(w))
		             	break
		  	 endif
			myList+=NameofWave(w) +";"
		 	i += 1
		while (1)
		
		n = ItemsInList(RemoveFromList(IndependentVariableNameStr, myList))
		
		Redimension/n=(n) TempTextWave
	
		List2TextWaveNoCreate(RemoveFromList(IndependentVariableNameStr, myList), TempTextWave)
		
		MyTempStr = RemoveItemsInListFromList(myList, MyWaveListStr) 
	
		ReOrderWaves_ReorganizeMatrix()
	endif
	
End


// *************************************************
// Checks column formatting and to see if parameters in top table are a matrix; creates individual waves if there is a matrix.

Function Create_ProcessFormatAsInTable(TableName)
	String TableName

	WAVE SelWaveMatrix= root:ICARTTFileFormat:SelWaveMatrix
	WAVE/T WaveListTextWaveMatrix= root:ICARTTFileFormat:WaveListTextWaveMatrix
	WAVE/T WaveListTextWave= root:ICARTTFileFormat:WaveListTextWave
	SVAR IndependentVariableNameStr= root:ICARTTFileFormat:IndependentVariableNameStr

	NVAR IsMatrixData = root:ICARTTFileFormat:IsMatrixData
	SVAR MatrixColNameList = root:ICARTTFileFormat:MatrixColNameList
	
	string myList, myWaveStr, myWaveInfo, myColInfo, formatStr, digitsStr	
	variable myIndex, myNum
	
	string colNameStr, colNameList=""
	variable numCols, numRows, i

	Create_ReOrderWavesFromTable()
	
	WAVE matrixWave = $WaveListTextWave[0]	
	
	numCols = DimSize(matrixWave,1)
	
	if (numCols > 0 )
		Print "// The variable you chose is a matrix - separating matrix into separate columns."
		IsMatrixData = 1
	
		myColInfo = TableInfo(TableName, 1)
		formatStr = StringByKey("FORMAT", myColInfo)
		digitsStr = StringByKey("DIGITS", myColInfo)
		WaveListTextWaveMatrix[myIndex-1][0]	= digitsStr

		numRows =DimSize(matrixWave,0)
		
		DoWindow/k IcarttMatrixDataTable
		edit/N=IcarttMatrixDataTable $IndependentVariableNameStr
		
		for(i=0; i< numCols; i+=1)
			colNameStr  = UniqueName(Nameofwave(matrixWave), 1, 0)			
			MatrixColNameList+=colNameStr+";"
			make/o/n=(numRows) $colNameStr
			WAVE colWave = $colNameStr
			colWave = matrixWave[p][i]
			AppendToTable colWave
			ModifyTable format(colWave)=str2num(formatStr)
			ModifyTable digits(colWave)=str2num(digitsStr)
		endfor 
		
		Create_ReOrderWavesFromTable()
		TableName = "IcarttMatrixDataTable"
	
	else
		IsMatrixData = 0
		MatrixColNameList=""
	endif
	
	myList = ""	
	myIndex=0

	do
	 	Wave/Z w = WaveRefIndexed(TableName, myIndex, 1)
	       if (!WaveExists(w))
	             	break
	        else
	             if (DimSize(w,1)>0)			// deal with 2d matrix case
	             		myIndex+=DimSize(w,1)
	             endif
	  	 endif
		myList+=NameofWave(w) +";"
	  	myIndex += 1
	while (1)
	
	myNum = ItemsInList(myList)
	
	myWaveStr = StringFromList(0, myList,";")
	
	if (cmpstr(IndependentVariableNameStr, myWaveStr)!=0) 	// the first column should be the independent parameter	
		Abort "Please make sure that the left most column is the independent parameter (timewave) - Aborting from ProcessFormatAsInTable"
	endif

	for (myIndex=1;myIndex<myNum; myIndex+=1)		// start after the first column, which should be the independent parameter column.

		myWaveStr = StringFromList(myIndex, myList,";")
		myColInfo = TableInfo(TableName,myIndex)
		formatStr = StringByKey("FORMAT", myColInfo)
		
		strswitch (formatStr)
			case "8": // date time format
				WaveListTextWaveMatrix[myIndex-1][0]	= "t"	
				break
			case "5": // exponential format
				WaveListTextWaveMatrix[myIndex-1][0]	= "e"	
				break	
			case "3": // decimal, no commas format
				digitsStr = StringByKey("DIGITS", myColInfo)
				WaveListTextWaveMatrix[myIndex-1][0]	= digitsStr
				break
			default :
				WaveListTextWaveMatrix[myIndex-1][0]	="1"		
		endswitch
		
		SelWaveMatrix[myIndex-1][0] = 3			// indicate that this column, wave was selected.

	endfor
	
	ControlUpdate/W=Icartt_Panel VariableNamesListBox

	DoWindow/K ReOrderWaveNamesPanel

End

// *************************************************
// The functions below are directly related to the Icartt panel


// *************************************************
// Creates root:ICARTTFileFormat:Panel data folder and sets globals here to be used by the icartt panel.

Function IcarttPanel_CreateGlobals()

	string  getFolderStr
	
	getFolderStr = GetDataFolder(1)
	
	NewDataFolder/S/O root:ICARTTFileFormat:Panel
	
	string/g useTableStr, useManualStr
	string/g fileNameInfoStr, fileLocInfoStr
	string/g dataIntervalInfoStr
	string/g alternateCreateFileStr, editParamStr, paramInfoStr
	string/g specialCommentsInfoStr, revCommentsInfoStr, otherCommentsInfoStr, normalCommentsInfoStr
	string/g icarttPathStr =""  //gets set via IcarttButtonFilePath or by loading in header info from an existing file.
	
	useTableStr = ""
	useTableStr += "* By clicking the 'Use Table Format' button an attempt will be made\r"
	useTableStr += "\tto read in the parameters and format from the top table.\r"
	useTableStr += "* After clicking the button, review items in the list box.\r"
	useTableStr += "* In the top table the first, left-most column in the top table must\r"
	useTableStr += "\tmatch the UTC wave selected in step 1.\r"
	useTableStr += "* Matrix (2-D wave) formatting is a special case.  One-dimensional\r"
	useTableStr += "\twaves such as mymatrix0,mymatrix1 will be generated."
	
	useManualStr = ""
	useManualStr += "* Use the shift key and mouse to make multiple selections.\r"
	useManualStr += "* Click the 'Set Order of Waves' button to reorder\r"
	useManualStr += "\tparameters in the list box below.\r" 
	useManualStr += "* If using 'e', exponential format, you can select below the\r"
	useManualStr += "\tnumber of digits after the decimal that will be printed.\r" 
	useManualStr += "* If working with a matrix (2- wave), create a table\r"
	useManualStr += "\tand use the table settings format.\r"

	fileNameInfoStr = ""
	fileNameInfoStr += "* Revision values 'A', 'B', ... = field data; Revision values '0', '1' ... = final data. This value \r"
	fileNameInfoStr += "\tmust equal that given in the REVISION field at the bottom of tab 'Header 2'.\r"
	fileNameInfoStr += "* The yyyymmdd must equal the date value of the first point in the independent parameter.\r"
	
	fileLocInfoStr = ""
	fileLocInfoStr +="* Directories are given on separate lines.\r"
	fileLocInfoStr +="* If blank, a prompt will appear when you\r"
	fileLocInfoStr +="\tclick a Create File button below.\r"
	fileLocInfoStr +="* Use the Change Icartt File Path button \r"
	fileLocInfoStr +="\tto create or change the path.\r"
	fileLocInfoStr +="* If a file with the name above exists in this\r"
	fileLocInfoStr +="\tlocation, it will be overwritten. \r"
	
	dataIntervalInfoStr = "Data Interval is in integer seconds.  Enter 0 for start-stop data."
	
	alternateCreateFileStr  = ""
	alternateCreateFileStr += "* These options are useful when you\r"
	alternateCreateFileStr += "\twant to test settings or create a\r"
	alternateCreateFileStr += "\ttemplate file for a specific project."
	
	editParamStr = ""
	editParamStr += "* To edit text beyond the window display select the radio button, then Edit.\r"
	editParamStr += "* Press Accept in this window when you are ready to transer the text\r"
	editParamStr += "\t - if you simply close the editing box, no text will be transfered.\r"
	editParamStr += "* Parameters with a bold 'R' will retain carriage return values, but\r"
	editParamStr += "\tblank lines will be eliminated.\r"
	editParamStr += "* Do not get fancy in formatting your comments  - do not use bold, italic, \r"
	editParamStr += "\tunderline, greek symbols, etc.; tabs do not translate well across platforms.\r"
	editParamStr += "* Feel free to include journal articles or web sites references if you want\r"
	editParamStr += "\tto provide more details than are possible in the header."
	
	paramInfoStr = ""
	paramInfoStr += "* The first column indicates the # of digits\r"
	paramInfoStr += "\tafter the decimal that will be written.\r"
	paramInfoStr += "* The timewave selected in step 1 will be the\r"
	paramInfoStr += "\tfirst column printed to the file and is not\r"
	paramInfoStr += "\tselectable in this list.\r"
	paramInfoStr += "\r"
	paramInfoStr +="* A 't' format in the first column indicates\r"
	paramInfoStr +="\ttime. The seconds from midnight of the start\r"
	paramInfoStr +="\tdate will be calculated.\r"
	paramInfoStr += "* An 'e' indicates exponential format.\r"
	paramInfoStr += "\tThe number of significant digits for this\r"
	paramInfoStr += "\tformat can be set individually in a table\r"
	paramInfoStr += "\tor globally using the variable above (Click\r"
	paramInfoStr += "\tUse Manual)."
	
	specialCommentsInfoStr = ""
	specialCommentsInfoStr += "Typically these comments should be reserved for identifying circumstances or\r"
	specialCommentsInfoStr += "treatment of data that is unique to this file.  (i.e. pump failed on this flight..etc)."
		
	revCommentsInfoStr = ""
	revCommentsInfoStr+="Revision comments are cumulative.  Comments for the latest revision should be\r"
	revCommentsInfoStr+="prepended to previous comments i.e. 'R1: fixed bad cal. R0: final data'."
	
	otherCommentsInfoStr=""
	otherCommentsInfoStr +="Typically these comments encompass any info the PI would like to include\r"
	otherCommentsInfoStr+="in every file, such as detailed sampling or anaylsis technical info."
	
	normalCommentsInfoStr = ""
	normalCommentsInfoStr +="* The same instructions from Step 3, part 1 apply.\r"
	normalCommentsInfoStr+="* FYI - The entries below constitute the 'Normal Comments' section in the icartt file."

	SetDataFolder getFolderStr

End

// *************************************************
// Creates the Icartt panel window and all the trappings.

Function IcarttPanel_CreatePanel() 
 
 	DoWindow/F ICARTT_Panel
	if(V_flag == 1)
		DoWindow/K ICARTT_Panel
	endif
		
	SVAR useTableStr = root:ICARTTFileFormat:Panel:useTableStr
	
	if (SVAR_Exists(useTableStr) ==0)
		IcarttPanel_CreateGlobals()
	endif

	SVAR IndependentVariableNameStr =root:ICARTTFileFormat:IndependentVariableNameStr
	
	SVAR MyWaveListStr =root:ICARTTFileFormat:MyWaveListStr
	WAVE/T WaveListTextWave = root:ICARTTFileFormat:WaveListTextWave
	WAVE/T WaveListTextWaveMatrix = root:ICARTTFileFormat:WaveListTextWaveMatrix
	WAVE SelWaveMatrix = root:ICARTTFileFormat:SelWaveMatrix
	
	if (  (WaveExists(WaveListTextWave)==0 ) || (WaveExists(WaveListTextWaveMatrix)==0 ) || (WaveExists(SelWaveMatrix)==0 )  )
		Abort "Aborting from Create_CreatePanel - a wave doesn't exist." + num2str(WaveExists(WaveListTextWave) )+ num2str(WaveExists(WaveListTextWaveMatrix))+ num2str(WaveExists(SelWaveMatrix) )  
	endif
	
	NVAR FileOpenAfterCreateVar =  root:ICARTTFileFormat:FileOpenAfterCreateVar
	NVAR FormatAsInTopTableVar =  root:ICARTTFileFormat:FormatAsInTopTableVar

	variable n
	
	// update the parameters.

	MyWaveListStr =WaveList("*",";","") 			// The 0th element in the string is always no in the WaveListTextWave
	
	n = ItemsInList(MyWaveListStr)

	Redimension/n=(n-1) WaveListTextWave
	WaveListTextWaveMatrix[][0] = "" // the number of digits after the decimal
	WaveListTextWaveMatrix[][1]	= ""
	
	SelWaveMatrix[][0] = 2		// First column, containing number of digits past decimal, will be editable
	SelWaveMatrix[][1] = 0
		
	FileOpenAfterCreateVar = 0
	FormatAsInTopTableVar = 0
	
	PauseUpdate; Silent 1		// building window...
	NewPanel /W=(373,318,911,849) as "ICARTT_Panel"
	DoWindow/C ICARTT_Panel
//	ShowTools
	SetDrawLayer UserBack
	SetDrawEnv fsize= 14,fstyle= 1,textrgb= (16385,16388,65535)
	DrawText 11,24,"ICARTT Format Helper v2.0"
	TabControl IcarttTabControl,pos={6,52},size={480,450},proc=IcarttTabProc_IcarttPanel
	TabControl IcarttTabControl,tabLabel(0)="Select Waves",tabLabel(1)="Header 1"
	TabControl IcarttTabControl,tabLabel(2)="Header 2",tabLabel(3)="Create File"
	TabControl IcarttTabControl,value= 0
	
	// tab 0 select waves
	TitleBox titleboxStep0,pos={20,80},size={311,17},title="Step 0. (Optional) Update the wave list"
	TitleBox titleboxStep0,fSize=14,frame=0,fStyle=1
	TitleBox titleboxStep1,pos={20,105},size={265,17},title="Step 1. Select the UTC time wave"
	TitleBox titleboxStep1,fSize=14,frame=0,fStyle=1
	TitleBox titleboxStep2,pos={20,130},size={390,17},title="Step 2. Select dependent parameters & format"
	TitleBox titleboxStep2,fSize=14,frame=0,fStyle=1
	Button buttonUpdateWaveList,pos={350,80},size={120,20},proc=IcarttButton_UpdateWavesList,title="Update Wave List"
	PopupMenu popUpIndependentVariableName,pos={350,105},size={78,20},proc=IcarttPopup_IndepVarName
	PopupMenu popUpIndependentVariableName,mode=1,popValue = "none", value= #"root:ICARTTFileFormat:MyWaveListStr"
	GroupBox groupBoxTab0,pos={20,182},size={455,118}
	CheckBox checkUseTable,pos={228,161},size={120,14},proc=IcarttCheck_ManualOrTable,title="Use Top Table Settings"
	CheckBox checkUseTable,value= 0,mode=1
	CheckBox checkUseManual,pos={85,161},size={108,14},proc=IcarttCheck_ManualOrTable,title="Use Manual Settings"
	CheckBox checkUseManual,value= 1,mode=1
	TitleBox titleboxUseTableInfo,pos={29,191},size={304,84},disable=1,frame=0
	TitleBox titleboxUseTableInfo,variable= root:ICARTTFileFormat:Panel:UseTableStr
	TitleBox titleboxUseManualInfo,pos={27,190},size={263,72},frame=0
	TitleBox titleboxUseManualInfo,variable= root:ICARTTFileFormat:Panel:UseManualStr
	Button buttonSetOrder,pos={316,190},size={150,20},proc=ReOrderWaves_CreatePanel,title="Set Order of Waves"
	Button buttonUseTableFormat,pos={349,192},size={120,20},disable=1,proc=IcarttButton_FormatViaTable,title="Format as in table"
	SetVariable setVarExponFormat,pos={127,278},size={340,15},title="When using exponential format, number of digits after decimal:"
	SetVariable setVarExponFormat,value= root:ICARTTFileFormat:ExponFormatVar
	ListBox listboxVariableNames,pos={251,317},size={218,162}
	ListBox listboxVariableNames,listWave=root:ICARTTFileFormat:WaveListTextWaveMatrix
	ListBox listboxVariableNames,selWave=root:ICARTTFileFormat:SelWaveMatrix,mode= 4
	ListBox listboxVariableNames,editStyle= 1,widths={10,100},proc=IcarttListBox_SelectParams
	TitleBox titleboxParameterFormatInfo,pos={21,318},size={218,176}
	TitleBox titleboxParameterFormatInfo,variable= root:ICARTTFileFormat:Panel:ParamInfoStr

	// tab 1 header part 1 
	TitleBox titleboxHeaderpart1,pos={20,80},size={50,20}, disable=1,title="Step 3, part 1.  Edit Header Info"
	TitleBox titleboxHeaderpart1,fSize=14,frame=0,fStyle=1
	TitleBox titleboxstep3InfoStr,pos={20,103},size={351,108},disable=1,frame=0
	TitleBox titleboxstep3InfoStr,variable= root:ICARTTFileFormat:Panel:editParamStr
	SetVariable setVarPINameStr,pos={22,249},size={420,15},disable=1,title="PI Name"
	SetVariable setVarPINameStr,value= root:ICARTTFileFormat:PINameStr
	SetVariable setVarPIAffiliationStr,pos={22,271},size={420,15},disable=1,title="PI Affiliation"
	SetVariable setVarPIAffiliationStr,value= root:ICARTTFileFormat:PIAffiliationStr
	SetVariable setVarDataSourceDescriptionStr,pos={22,293},size={420,15},disable=1,title="Data Source Description"
	SetVariable setVarDataSourceDescriptionStr,value= root:ICARTTFileFormat:DataSourceDescriptionStr
	SetVariable setVarMissionNameStr,pos={22,317},size={420,15},disable=1,title="Mission Name"
	SetVariable setVarMissionNameStr,value= root:ICARTTFileFormat:MissionNameStr
	SetVariable setVarUTCRevisionDateStr,pos={22,339},size={420,15},disable=1,title="Revision Date In yyyy mm dd"
	SetVariable setVarUTCRevisionDateStr,value= root:ICARTTFileFormat:UTCRevisionDateStr
	SetVariable setVarDataIntervalVar,pos={22,363},size={420,15},disable=1,title="Data Interval"
	SetVariable setVarDataIntervalVar,limits={0,inf,1},value= root:ICARTTFileFormat:DataIntervalVar
	SetVariable setVarSpecialCommentsStr,pos={22,403},size={420,15},disable=1,title="Special Comments"
	SetVariable setVarSpecialCommentsStr,value= root:ICARTTFileFormat:SpecialCommentsStr
	TitleBox titleboxDataIntervalInfo,pos={154,381},size={288,12},disable=1,frame=0
	TitleBox titleboxDataIntervalInfo,variable= root:ICARTTFileFormat:Panel:dataIntervalInfoStr
	TitleBox titleboxSpecialCommentInfo,pos={77,420},size={360,24},disable=1,frame=0
	TitleBox titleboxSpecialCommentInfo,variable= root:ICARTTFileFormat:Panel:SpecialCommentsInfoStr
	TitleBox titleboxSpecialCommentsR,pos={463,405},size={9,17},disable=1,title="R"
	TitleBox titleboxSpecialCommentsR,fSize=14,frame=0,fStyle=1
	CheckBox radioPIAffiliation,pos={445,269},size={16,14},disable=1,proc=IcarttCheck_RadioComments,title=""
	CheckBox radioPIAffiliation,value= 0,mode=1
	CheckBox radioDataSourceDescription,pos={445,293},size={16,14},disable=1,proc=IcarttCheck_RadioComments,title=""
	CheckBox radioDataSourceDescription,value= 0,mode=1
	CheckBox radioSpecialComments,pos={445,406},size={16,14},disable=1,proc=IcarttCheck_RadioComments,title=""
	CheckBox radioSpecialComments,value= 0,mode=1

	Button buttonEditParamStr,pos={426,81},size={50,20},disable=1,proc=IcarttButton_EditParam,title="Edit"
	Button buttonAcceptParamStr,pos={427,107},size={50,20},disable=1,proc=IcarttButton_AcceptParam,title="Accept"

	// tab 1 header part 2 
	TitleBox titleboxHeaderpart2,pos={20,80},size={250,17},disable=1,title="Step 3, part 2. Edit Header Info"
	TitleBox titleboxHeaderpart2,fSize=14,frame=0,fStyle=1
	TitleBox titleboxNormalCommentsInfo,pos={17,103},size={358,24},disable=1,frame=0
	TitleBox titleboxNormalCommentsInfo,variable= root:ICARTTFileFormat:Panel:NormalCommentsInfoStr
	
	SetVariable setVarPIContactInfoStr,pos={20,147},size={420,15},disable=1,title="PI_CONTACT_INFO"
	SetVariable setVarPIContactInfoStr,value= root:ICARTTFileFormat:PIContactInfoStr
	SetVariable setVarPlatformSiteInfoStr,pos={20,167},size={420,15},disable=1,title="PLATFORM"
	SetVariable setVarPlatformSiteInfoStr,value= root:ICARTTFileFormat:PlatformSiteInfoStr
	SetVariable setVarLocationInfoStr,pos={20,187},size={420,15},disable=1,title="LOCATION"
	SetVariable setVarLocationInfoStr,value= root:ICARTTFileFormat:LocationInfoStr
	SetVariable setVarAssociatedFileNamesStr,pos={20,207},size={420,15},disable=1,title="ASSOCIATED_DATA"
	SetVariable setVarAssociatedFileNamesStr,value= root:ICARTTFileFormat:AssociatedFileNamesStr
	SetVariable setVarInstrumentInfoStr,pos={20,227},size={420,15},disable=1,title="INSTRUMENT_INFO"
	SetVariable setVarInstrumentInfoStr,value= root:ICARTTFileFormat:InstrumentInfoStr
	SetVariable setVarDataInfoStr,pos={20,247},size={420,15},disable=1,title="DATA_INFO"
	SetVariable setVarDataInfoStr,value= root:ICARTTFileFormat:DataInfoStr
	SetVariable setVarUncertaintyInfoStr,pos={20,267},size={420,15},disable=1,title="UNCERTAINTY"
	SetVariable setVarUncertaintyInfoStr,value= root:ICARTTFileFormat:UncertaintyInfoStr
	SetVariable setVarULODValueStr,pos={20,287},size={420,15},disable=1,title="ULOD_VALUE"
	SetVariable setVarULODValueStr,value= root:ICARTTFileFormat:ULODValueStr
	SetVariable setVarLLODValueStr,pos={20,307},size={420,15},disable=1,title="LLOD_VALUE"
	SetVariable setVarLLODValueStr,value= root:ICARTTFileFormat:LLODValueStr
	SetVariable setVarDataManagerContactInfoStr,pos={20,326},size={420,15},disable=1,title="DM_CONTACT_INFO"
	SetVariable setVarDataManagerContactInfoStr,value= root:ICARTTFileFormat:DataManagerContactInfoStr
	SetVariable setVarProjectInfoStr,pos={20,348},size={420,15},disable=1,title="PROJECT_INFO"
	SetVariable setVarProjectInfoStr,value= root:ICARTTFileFormat:ProjectInfoStr
	SetVariable setVarStipulationsOnUseStr,pos={19,368},size={420,15},disable=1,title="STIPULATIONS_ON_USE"
	SetVariable setVarStipulationsOnUseStr,value= root:ICARTTFileFormat:StipulationsOnUseStr
	SetVariable setVarOtherCommentsStr,pos={19,388},size={420,15},disable=1,title="OTHER_COMMENTS"
	SetVariable setVarOtherCommentsStr,value= root:ICARTTFileFormat:OtherCommentsStr
	SetVariable setVarRevisionNumberStr,pos={20,435},size={420,15},disable=1,title="REVISION"
	SetVariable setVarRevisionNumberStr,value= root:ICARTTFileFormat:RevisionNumberStr
	SetVariable setVarRevisionStr,pos={21,456},size={420,15},disable=1,title="Rev Comments"
	SetVariable setVarRevisionStr,value= root:ICARTTFileFormat:RevisionStr
	TitleBox titleboxOtherCommentsStr,pos={105,405},size={330,24},disable=1,frame=0
	TitleBox titleboxOtherCommentsStr,variable= root:ICARTTFileFormat:Panel:OtherCommentsInfoStr
	TitleBox titleboxRevCommentsInfo,pos={77,473},size={358,24},disable=1,frame=0
	TitleBox titleboxRevCommentsInfo,variable= root:ICARTTFileFormat:Panel:RevCommentsInfoStr
	CheckBox radioPIContactInfo,pos={444,148},size={16,14},disable=1,proc=IcarttCheck_RadioComments,title=""
	CheckBox radioPIContactInfo,value= 0,mode=1
	CheckBox radioLocationInfo,pos={444,188},size={16,14},disable=1,proc=IcarttCheck_RadioComments,title=""
	CheckBox radioLocationInfo,value= 0,mode=1
	CheckBox radioInstrumentInfo,pos={444,227},size={16,14},disable=1,proc=IcarttCheck_RadioComments,title=""
	CheckBox radioInstrumentInfo,value= 0,mode=1
	CheckBox radioDataInfo,pos={444,247},size={16,14},disable=1,proc=IcarttCheck_RadioComments,title=""
	CheckBox radioDataInfo,value= 0,mode=1
	CheckBox radioUncertaintyInfo,pos={444,268},size={16,14},disable=1,proc=IcarttCheck_RadioComments,title=""
	CheckBox radioUncertaintyInfo,value= 0,mode=1
	CheckBox radioStipulationsOnUse,pos={444,369},size={16,14},disable=1,proc=IcarttCheck_RadioComments,title=""
	CheckBox radioStipulationsOnUse,value= 0,mode=1
	CheckBox radioOtherComments,pos={444,389},size={16,14},disable=1,proc=IcarttCheck_RadioComments,title=""
	CheckBox radioOtherComments,value= 1,mode=1
	CheckBox radioRevisionComments,pos={445,457},size={16,14},disable=1,proc=IcarttCheck_RadioComments,title=""
	CheckBox radioRevisionComments,value= 0,mode=1
	TitleBox titleboxOtherCommentsR,pos={460,386},size={9,17},disable=1,title="R"
	TitleBox titleboxOtherCommentsR,fSize=14,frame=0,fStyle=1
	TitleBox titleboxRevisionCommentsR,pos={460,453},size={9,17},disable=1,title="R"
	TitleBox titleboxRevisionCommentsR,fSize=14,frame=0,fStyle=1
	// tab 3  create file	
	
	TitleBox titleboxStep4,pos={20,80},size={161,17},disable=1,title="Step 4. Create a file"
	TitleBox titleboxStep4,fSize=14,frame=0,fStyle=1
	
	GroupBox groupFileName,pos={20,100},size={451,75},disable=1,title="File Name"
	GroupBox groupFileName,fStyle=1
	SetVariable setVarFileName,pos={30,116},size={430,15},disable=1,title="DataID_Platform_yyyymmdd_Rx_(OptionalMerge).ict"
	SetVariable setVarFileName,value= root:ICARTTFileFormat:FileNameStr
	TitleBox titleboxFileNameInfo,pos={30,133},size={417,48},disable=1,frame=0
	TitleBox titleboxFileNameInfo,fStyle=0
	TitleBox titleboxFileNameInfo,variable= root:ICARTTFileFormat:Panel:fileNameInfoStr
	
	GroupBox groupFileLocation,pos={20,179},size={400,150},disable=1,title="File Location"
	GroupBox groupFileLocation,fStyle=1
	TitleBox titleboxFileLocationInfo,pos={30,195},size={188,60},disable=1,frame=0
	TitleBox titleboxFileLocationInfo,variable= root:ICARTTFileFormat:Panel:fileLocInfoStr
	TitleBox titleboxIcarttPath,pos={234,196},size={118,116},disable=1
	TitleBox titleboxIcarttPath,variable= root:ICARTTFileFormat:Panel:icarttpathstr
	Button buttonIcarttFilePath,pos={30,290},size={180,20},disable=1,proc=IcarttButton_IcarttFilePath,title="Change Icartt File Location"

	CheckBox checkFileOpenAfterCreate,pos={25,333},size={290,14},disable=1,proc=IcarttCheck_OpenAfterCreate,title="Open and view file after creation? (File will not be editable.)"
	CheckBox checkFileOpenAfterCreate,variable= root:ICARTTFileFormat:FileOpenAfterCreateVar
	
	Button buttonCreate,pos={25,357},size={180,100},disable=1,proc=IcarttButton_CreateFile,title="Create ICARTT File"

	GroupBox groupCreateFileAlternatives,pos={216,350},size={206,111},disable=1,title="Alternatives"
	TitleBox titleboxAlternateFileCreate,pos={233,366},size={168,36},disable=1,frame=0
	TitleBox titleboxAlternateFileCreate,variable= root:ICARTTFileFormat:Panel:AlternateCreateFileStr
	Button buttonCreateHeader,pos={228,409},size={180,20},disable=1,proc=IcarttButton_CreateHead,title="Create Header File Only"
	Button buttonCreateData,pos={228,434},size={180,20},disable=1,proc=IcarttButton_CreateData,title="Create Data File Only"

	TitleBox titleboxStep5,pos={26,469},size={324,17},disable=1,title="Step 5 (Optional). Close ICARTT_Panel ->"
	TitleBox titleboxStep5,fSize=14,frame=0,fStyle=1
	Button buttonExit,pos={360,469},size={50,20},disable=1,proc=IcarttButton_Exit,title="Exit"

End

// *************************************************
// Dictates which controls are displayed (disabled) when the user selects a tab.

Function IcarttTabProc_IcarttPanel(ctrlname, tabNum) : TabControl
	String ctrlName
	Variable tabNum

	variable isTab0 = (tabnum==0)
	variable isTab1 = (tabnum==1)
	variable isTab2 = (tabnum==2)
	variable isTab3 = (tabnum==3)
	
	NVAR FormatAsInTopTableVar = root:ICARTTFileFormat:FormatAsInTopTableVar
	
	// hide or show tab0 - Select Waves
	
	Checkbox checkUseTable, value = FormatAsInTopTableVar
	Checkbox checkUseManual, value = !FormatAsInTopTableVar
	
	ModifyControlList "buttonUpdateWaveList;" disable = !isTab0 // hide if not Tab0
	ModifyControlList "popupIndependentVariableName;" disable = !isTab0 // hide if not Tab0
	ModifyControlList "checkUseManual;checkUseTable" disable = !isTab0 
	ModifyControlList "listboxVariableNames" disable = !isTab0 
	ModifyControlList "groupboxTab0" disable = !isTab0 
	
	TitleBox titleboxStep0 disable= !isTab0 
	TitleBox titleboxStep1 disable= !isTab0 
	TitleBox titleboxStep2 disable= !isTab0 
	TitleBox titleboxParameterFormatInfo disable = !isTab0 
	
	ModifyControlList "buttonUseTableFormat" disable = ( !isTab0 ||  !FormatAsInTopTableVar ) 
	ModifyControlList "buttonSetOrder;setVarExponFormat" disable = (!isTab0 || FormatAsInTopTableVar ) 
	
	TitleBox titleboxUseTableInfo disable = ( !isTab0  || !FormatAsInTopTableVar ) 
	TitleBox titleboxUseManualInfo disable = (!isTab0  || FormatAsInTopTableVar) 

	// hide or show tab1 - Header 1
	
	ModifyControlList "setVarPINameStr;setVarPIAffiliationStr;setVarDataSourceDescriptionStr;" disable = !isTab1
	ModifyControlList "setVarMissionNameStr;setVarUTCRevisionDateStr;setVarDataIntervalVar;setVarSpecialCommentsStr" disable = !isTab1
	
	ModifyControlList "buttonEditParamStr;buttonAcceptParamStr" disable = !isTab1

	ModifyControlList "radioPIAffiliation;radioDataSourceDescription;radioSpecialComments" disable = !isTab1

	TitleBox titleboxHeaderpart1 disable = !isTab1
	TitleBox titleboxHeader1Part1 disable = !isTab1
	TitleBox titleboxHeader1Part2 disable = !isTab1
	TitleBox titleboxDataIntervalInfo disable = !isTab1
	TitleBox titleboxSpecialCommentInfo disable = !isTab1
	 
	TitleBox titleboxstep3InfoStr disable = !isTab1

	Titlebox titleboxSpecialCommentsR disable = !isTab1

	// hide or show tab2 - Header 2

	ModifyControlList "setVarPIContactInfoStr;setVarPlatformSiteInfoStr;setVarLocationInfoStr" disable = !isTab2
	ModifyControlList "setVarAssociatedFileNamesStr;setVarInstrumentInfoStr;setVarDataInfoStr" disable = !isTab2
	ModifyControlList "setVarUncertaintyInfoStr;setVarULODValueStr;setVarLLODValueStr" disable = !isTab2
	ModifyControlList "setVarDataManagerContactInfoStr;setVarProjectInfoStr;setVarStipulationsOnUseStr" disable = !isTab2
	ModifyControlList "setVarOtherCommentsStr;setVarRevisionNumberStr;setVarRevisionStr" disable = !isTab2
	
	TitleBox titleboxHeaderpart2 disable = !isTab2

	TitleBox titleboxRevCommentsInfo disable = !isTab2
	TitleBox titleboxOtherCommentsStr disable = !isTab2

	TitleBox titleboxNormalCommentsInfo disable = !isTab2

	ModifyControlList "radioPIContactInfo;radioLocationInfo;radioInstrumentInfo;radioDataInfo;radioUncertaintyInfo;"  disable = !isTab2
	ModifyControlList "radioStipulationsOnUse;radioOtherComments;radioRevisionComments" disable = !isTab2

	Titlebox titleboxOtherCommentsR disable = !isTab2
	TitleBox titleboxRevisionCommentsR disable = !isTab2

	// hide or show tab1or2
	
	ModifyControlList "buttonEditParamStr;buttonAcceptParamStr" disable = (!isTab2 && !isTab1)

	// hide or show tab3 - Create File

	ModifyControlList "checkFileOpenAfterCreate" disable = !isTab3
	ModifyControlList "buttonCreate;buttonCreateHeader;buttonCreateData" disable = !isTab3

	TitleBox titleboxStep4 disable= !isTab3

	ModifyControlList "buttonIcarttFilePath" disable = !isTab3
	ModifyControlList "groupFileName;groupFileLocation;groupCreateFileAlternatives" disable = !isTab3
	
	TitleBox titleboxAlternateFileCreate  disable= !isTab3

	ModifyControlList "setVarFileName;buttonExit;titleboxFileNameInfo" disable = !isTab3

	TitleBox titleboxFileLocationInfo disable= !isTab3
	TitleBox titleboxIcarttPath disable= !isTab3
	
	TitleBox titleboxStep5 disable= !isTab3

End


// *************************************************
//  Looks in the current data folder for current wave names.

Function IcarttButton_UpdateWavesList(ctrlName): ButtonControl
	String ctrlName
	
	PauseUpdate; Silent 1		// building window...
		
	SVAR MyTempStr = root:ICARTTFileFormat:MyTempStr
	SVAR IndependentVariableNameStr = root:ICARTTFileFormat:IndependentVariableNameStr
	SVAR UTCDataStartStr= root:ICARTTFileFormat:UTCDataStartStr
	NVAR UTCDataStartVar = root:ICARTTFileFormat:UTCDataStartVar
	SVAR MyWaveListStr= root:ICARTTFileFormat:MyWaveListStr

	WAVE/T WaveListTextWaveMatrix = root:ICARTTFileFormat:WaveListTextWaveMatrix
	WAVE/T WaveListTextWave = root:ICARTTFileFormat:WaveListTextWave
	WAVE SelWaveMatrix= root:ICARTTFileFormat:SelWaveMatrix
	WAVE/T TempTextWave = root:ICARTTFileFormat:TempTextWave
					
	MyWaveListStr = WaveList("*", ";", "")
	variable n = ItemsInList(MyWaveListStr)
	
	if (n==0)
		Abort "There are no igor waves in the experiment. - Aborting from IcarttButton_UpdateWavesList"
	else
		Redimension/n=(n-1) WaveListTextWave				// for ListBox 
		Redimension/n=(n-1,2) WaveListTextWaveMatrix		//  ListBox; 1st Column is # of decimal place, 2nd is for names
		Redimension/n=(n-1,2) SelWaveMatrix				// used with ListBox; 1st Column is # of decimal place, 2nd is for names
	endif
	
	if (FindListItem(IndependentVariableNameStr, MyWaveListStr) >=0)
	
		List2TextWaveNoCreate(RemoveFromList(IndependentVariableNameStr, MyWaveListStr ), WaveListTextWave)
	
		WaveListTextWaveMatrix[][0] = "1" // the default number of digits after the decimal
		WaveListTextWaveMatrix[][1]= WaveListTextWave[p]
	
		SelWaveMatrix[][0] = 2		// First column, containing number of digits past decimal, will be editable
		SelWaveMatrix[][1] = 0	
	
		MyTempStr = RemoveFromList(IndependentVariableNameStr, MyWaveListStr )	
		Redimension/n=0 TempTextWave
	endif
		
	ControlUpdate/W=ICARTT_Panel popUpIndependentVariableName
	ControlUpdate/W=ICARTT_Panel listboxVariableNames

End

// *************************************************
// Select the independent variable wave; removes this wave from selection as a dependent wave and sets globals.
// Users often forget to do this step first.

Function IcarttPopup_IndepVarName(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	
	SVAR IndependentVariableNameStr = root:ICARTTFileFormat:IndependentVariableNameStr
	SVAR UTCDataStartStr= root:ICARTTFileFormat:UTCDataStartStr
	NVAR UTCDataStartVar = root:ICARTTFileFormat:UTCDataStartVar
	SVAR MyWaveListStr= root:ICARTTFileFormat:MyWaveListStr

	WAVE/T WaveListTextWaveMatrix = root:ICARTTFileFormat:WaveListTextWaveMatrix
	WAVE/T WaveListTextWave = root:ICARTTFileFormat:WaveListTextWave
	WAVE SelWaveMatrix= root:ICARTTFileFormat:SelWaveMatrix
		
	variable i, n
	
	WAVE twave = $popStr
	if (WaveExists(twave) ==0)
		Abort "Something is wrong with the independent variable - Aborting from IcarttPopup_IndepVarName"
	endif
	
	IndependentVariableNameStr = popStr

	i= twave[0] - mod( twave[0] , 86400)

	if ( (numtype (i)!=0) || (i  < date2secs(1980,1,1) )   )// 1980 is arbitrary
		Abort "The independent variable doesn't have a decent time value - Aborting from IcarttPopup_IndepVarName"
	endif
	
	UTCDataStartVar = i
	UTCDataStartStr = time2yyyymmddhhmmss(UTCDataStartVar)
	UTCDataStartStr = UTCDataStartStr[0,3]+ " " + UTCDataStartStr[4,5]+ " " + UTCDataStartStr[6,7]

	MyWaveListStr = WaveList("*", ";", "")
	
	n = ItemsInList(MyWaveListStr)

	Redimension/n=(n-1) WaveListTextWave		// for ListBox 
	Redimension/n=(n-1,2) WaveListTextWaveMatrix		//  ListBox; 1st Column is # of decimal place, 2nd is for names
	Redimension/n=(n-1,2) SelWaveMatrix		// used with ListBox; 1st Column is # of decimal place, 2nd is for names

	List2TextWaveNoCreate(RemoveFromList(popstr, MyWaveListStr ), WaveListTextWave)
	
	Icartt_Reorganize2DTextMatrix(WaveListTextWaveMatrix, WaveListTextWave)
	
	SelWaveMatrix[][0] = 2
	SelWaveMatrix[][1] = 0
	
	ControlUpdate/W=ICARTT_Panel VariableNamesListBox
	
End


// *************************************************
// Modifies display of options in Icartt panel.

Function IcarttCheck_ManualOrTable(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	NVAR FormatAsInTopTableVar = root:ICARTTFileFormat:FormatAsInTopTableVar
	
	strswitch(ctrlName)
	case "checkUseTable":
		FormatAsInTopTableVar = 1
		break
	case  "checkUseManual" :	
		FormatAsInTopTableVar = 0
		break
	endswitch

	Checkbox checkUseTable, value = FormatAsInTopTableVar
	Checkbox checkUseManual, value = !FormatAsInTopTableVar

	ModifyControlList "buttonUseTableFormat" disable = (!FormatAsInTopTableVar ) 
	ModifyControlList "buttonSetOrder;setVarExponFormat" disable = (FormatAsInTopTableVar ) 

	TitleBox titleBoxUseTableInfo disable = (!FormatAsInTopTableVar ) 
	TitleBox titleBoxUseManualInfo disable = (FormatAsInTopTableVar) 

//	ListBox listboxVariableNames mode = (4*  !FormatAsInTopTableVar)  // disallow users to edit window if in table mode

End


// *************************************************
// Creates a new panel or brings to the front the existing panel which assists the user to organize display of waves.

Function ReOrderWaves_CreatePanel(ctrlName) : ButtonControl
	String ctrlName

	PauseUpdate; Silent 1		// building window...
	
	DoWindow/F ReOrderWaveNamesPanel
	if(V_flag == 1)
		DoWindow/K ReOrderWaveNamesPanel
	endif
	
	SVAR MyTempStr = root:ICARTTFileFormat:MyTempStr
	SVAR IndependentVariableNameStr = root:ICARTTFileFormat:IndependentVariableNameStr
	SVAR UTCDataStartStr= root:ICARTTFileFormat:UTCDataStartStr
	NVAR UTCDataStartVar = root:ICARTTFileFormat:UTCDataStartVar
	SVAR MyWaveListStr= root:ICARTTFileFormat:MyWaveListStr

	WAVE/T WaveListTextWaveMatrix = root:ICARTTFileFormat:WaveListTextWaveMatrix
	WAVE/T WaveListTextWave = root:ICARTTFileFormat:WaveListTextWave
	WAVE SelWaveMatrix= root:ICARTTFileFormat:SelWaveMatrix
	WAVE/T TempTextWave = root:ICARTTFileFormat:TempTextWave
		
	if (Exists(IndependentVariableNameStr)!=1)
		Abort " You must choose an independent variable first- Aborting from ReOrderWaves_CreatePanel"
	endif		
	
	ControlInfo/W=ICARTT_Panel popUpIndependentVariableName
	if (cmpstr(S_Value, IndependentVariableNameStr)!=0)
		Abort " You must choose an independent variable first - Aborting from ReOrderWaves_CreatePanel"
	endif
			
	MyWaveListStr = WaveList("*", ";", "")
	variable n = ItemsInList(MyWaveListStr)
	if (n==0)
		Abort "Please load some igor waves into the experiment - Aborting from ReOrderWaves_CreatePanel"
	else
		Redimension/n=(n-1) WaveListTextWave		// for ListBox 
		Redimension/n=(n-1,2) WaveListTextWaveMatrix		//  ListBox; 1st Column is # of decimal place, 2nd is for names
		Redimension/n=(n-1,2) SelWaveMatrix		// used with ListBox; 1st Column is # of decimal place, 2nd is for names
	endif
		
	List2TextWaveNoCreate(RemoveFromList(IndependentVariableNameStr, MyWaveListStr ), WaveListTextWave)

	WaveListTextWaveMatrix[][0] = "1" // the default number of digits after the decimal
	WaveListTextWaveMatrix[][1]	= WaveListTextWave[p]
	
	SelWaveMatrix[][0] = 2		// First column, containing number of digits past decimal, will be editable
	SelWaveMatrix[][1] = 0	
	
	MyTempStr = RemoveFromList(IndependentVariableNameStr, MyWaveListStr )	
	Redimension/n=0 TempTextWave
	
	NewPanel /W=(264,175,696,405)
	DoWindow/C ReOrderWaveNamesPanel
	ShowTools
	SetDrawLayer UserBack

	DrawText 266,47,"Current Order"
	DrawText 8,172,"You do not have to order all the waves in the experiment."
	DrawText 4,189," The waves to the right will appear first in the Dependent"
	DrawText 9,206,"Parameter Box in the Create ICARTT File Panel."
	DrawText 9,225,"Waves not shown will follow."

	Button Exitbutton,pos={9,28},size={150,20},proc=ReOrderWaves_NoChange,title="Exit  - don't change"
	Button ReOrderWaveNamesButton,pos={9,4},size={150,20},proc=ReOrderWaves_Finish,title="Exit - I'm finished"
	Button AddButton,pos={15,76},size={200,20},proc=ReOrderWaves_AddWaveNames,title="Add this to chosen order"
	Button StartOver,pos={255,3},size={100,20},proc=ReOrderWaves_StartOver,title="Start over"

	PopupMenu MyWavespopup,pos={31,102},size={136,20},title="select wave "
	PopupMenu MyWavespopup,mode=1,popvalue="Select here",value= #"root:ICARTTFileFormat:MyTempStr"

	ListBox ThisOrderListBox,pos={226,51},size={167,93}
	ListBox ThisOrderListBox,listWave=root:ICARTTFileFormat:TempTextWave

End


// *************************************************
// Kills window ReOrderWaveNamesPanel.

Function ReOrderWaves_NoChange(ctrlName) : ButtonControl
	String ctrlName

	DoWindow/K ReOrderWaveNamesPanel	
End


// *************************************************
// Ends user manipulation of ReOrderWaveNamesPanel.

Function ReOrderWaves_Finish(ctrlName) : ButtonControl
	String ctrlName

	ReOrderWaves_ReorganizeMatrix()
End


// *************************************************
// Adds wave name to existing reorganized wave name list in ReOrderWaveNamesPanel.

Function ReOrderWaves_AddWaveNames(ctrlName) : ButtonControl
	String ctrlName

	SVAR MyTempStr = root:ICARTTFileFormat:MyTempStr

	WAVE/T TempTextWave = root:ICARTTFileFormat:TempTextWave

	string newOneStr
	variable  NumPtsTempTextWave = numpnts(TempTextWave)
	
	NumPtsTempTextWave = numpnts(TempTextWave)
	
	ControlInfo MyWavespopup
	newOneStr = S_Value
	MyTempStr = RemoveFromList(newOneStr,MyTempStr )
	
	InsertPoints NumPtsTempTextWave, 1, TempTextWave
	TempTextWave[NumPtsTempTextWave] = newOneStr

	ControlUpdate/W=ReOrderWaveNamesPanel ThisOrderListBox
	ControlUpdate/W=ReOrderWaveNamesPanel MyWavespopup

End


// *************************************************
//  Implements wave name organizing and sorting from the ReOrderWaveNamesPanel.

Function ReOrderWaves_ReorganizeMatrix()

	SVAR MyTempStr = root:ICARTTFileFormat:MyTempStr
	WAVE/T TempTextWave = root:ICARTTFileFormat:TempTextWave

	WAVE/T WaveListTextWaveMatrix = root:ICARTTFileFormat:WaveListTextWaveMatrix
	WAVE/T WaveListTextWave = root:ICARTTFileFormat:WaveListTextWave
	WAVE SelWaveMatrix= root:ICARTTFileFormat:SelWaveMatrix

	variable i, n, y, m
	string str
	
	duplicate/o/t WaveListTextWaveMatrix TempWaveListTextWaveMatrix
	duplicate/o/t WaveListTextWave TempWaveListTextWave
	
	TempWaveListTextWaveMatrix[][] = "x"		// just make sure they get changed below.
	TempWaveListTextWave = "y"
	
	n = numpnts(TempTextWave)	
	m = ItemsInlist(MyTempStr)

	for( i=0; i<n; i+=1 )
		str = TempTextWave[i]
		y = TextInTextwave(str, WaveListTextWave)
		TempWaveListTextWave[i] = str
		TempWaveListTextWaveMatrix[i][] = WaveListTextWaveMatrix[y][q]
//		print i, y,  str, TempWaveListTextWaveMatrix[i][0] , TempWaveListTextWaveMatrix[i][1] 
	endfor
		
	for(i=n; i<(n+m); i+=1)
		str = StringFromList(i-n, MyTempStr, ";")
		y = TextInTextwave(str, WaveListTextWave)
		TempWaveListTextWave[i] = str
		TempWaveListTextWaveMatrix[i][] = WaveListTextWaveMatrix[y][q]
//		print i, y, str, TempWaveListTextWaveMatrix[i][0] , TempWaveListTextWaveMatrix[i][1] 
	endfor
		
	SelWaveMatrix[][0] = 2
	SelWaveMatrix[][1] = 0

	WaveListTextWave = TempWaveListTextWave
	WaveListTextWaveMatrix = TempWaveListTextWaveMatrix[p][q]

	Killwaves TempWaveListTextWaveMatrix, TempWaveListTextWave
	
	ControlUpdate/W=ICARTT_Panel listboxVariableNames

	DoWindow/K ReOrderWaveNamesPanel

End


// *************************************************
// Resets ReOrderWaveNamesPanel to a default beginning state.

Function ReOrderWaves_StartOver(ctrlName) : ButtonControl
	String ctrlName

	SVAR MyWaveListStr= $"root:ICARTTFileFormat:MyWaveListStr"
	SVAR MyTempStr = $"root:ICARTTFileFormat:MyTempStr"
	SVAR IndependentVariableNameStr = $"root:ICARTTFileFormat:IndependentVariableNameStr"
	WAVE/T TempTextWave = $"root:ICARTTFileFormat:TempTextWave"
	
	MyTempStr =  RemoveFromList(IndependentVariableNameStr, MyWaveListStr )	

	Redimension/N=0 TempTextWave

	ControlUpdate/W=ReOrderWaveNamesPanel ThisOrderListBox
	ControlUpdate/W=ReOrderWaveNamesPanel MyWavespopup
End


// *************************************************
// Calls other functions to do the work in implementing the format via table option.

Function IcarttButton_FormatViaTable(ctrlName) : ButtonControl
	String ctrlName
	
	string tableName
	tableName = Create_CheckFormatAsInTable()
	if (strlen(tableName)>0)
		print "// Formatting via settings in Table "+tableName
		Create_ProcessFormatAsInTable(tableName)
	endif
	
End


// *************************************************
// Sets globals when user selects items in list box manually

Function IcarttListBox_SelectParams(ctrlName,row,col,event) : ListBoxControl
	String ctrlName
	Variable row
	Variable col
	Variable event

	SVAR VariableNamesList = root:ICARTTFileFormat:VariableNamesList
	NVAR NumVariablesVar = root:ICARTTFileFormat:NumVariablesVar
	
	VariableNamesList =  Create_CreateListVarStr()
	NumVariablesVar = ItemsInList(VariableNamesList, ";")

End


// *************************************************
// Creates an icartt formatted file.

Function IcarttButton_CreateFile(ctrlName) : ButtonControl
	String ctrlName
		
	NVAR FileOpenAfterCreateVar =  $"root:ICARTTFileFormat:FileOpenAfterCreateVar"
	SVAR FileNameStr =  $"root:ICARTTFileFormat:FileNameStr"
	SVAR FileNameStr4Macs =  $"root:ICARTTFileFormat:FileNameStr4Macs"
		
	Create_CreateHeader()
	Create_AppendData2Header()
	
	If (FileOpenAfterCreateVar==1)
		OpenNotebook/r/k=1/p=ICARTTFilePath	 FileNameStr	
	endif

End


// *************************************************
//  Creates and icartt formatted header file.

Function IcarttButton_CreateHead(ctrlName) : ButtonControl
	String ctrlName

	NVAR FileOpenAfterCreateVar =  root:ICARTTFileFormat:FileOpenAfterCreateVar
	SVAR FileNameStr =  root:ICARTTFileFormat:FileNameStr
		
	Create_CreateHeader()

	If (FileOpenAfterCreateVar==1)
		OpenNotebook/n=ICARTTHeaderFile/r/k=1/p=ICARTTFilePath FileNameStr
	endif
End


// *************************************************
//  Creates and icartt formatted data file with no header.

Function IcarttButton_CreateData(ctrlName) : ButtonControl
	String ctrlName

	SVAR FileNameStr = $"root:ICARTTFileFormat:FileNameStr"
	NVAR FileOpenAfterCreateVar =  $"root:ICARTTFileFormat:FileOpenAfterCreateVar"
	
	Create_AppendData2Header()

	If (FileOpenAfterCreateVar==1)
		OpenNotebook/r/k=1/p=ICARTTFilePath	 FileNameStr	
	endif

End


// *************************************************
// Sets global variable FileOpenAfterCreateVar.

Function IcarttCheck_OpenAfterCreate(name, value) : CheckBoxControl
	String name
	Variable value

	NVAR FileOpenAfterCreateVar = root:ICARTTFileFormat:FileOpenAfterCreateVar
	FileOpenAfterCreateVar = value

End


// *************************************************
// Prompts user to select a new icartt file path.

Function IcarttButton_IcarttFilePath(ctrlName) : ButtonControl
	String ctrlName
	
	SVAR IcarttPathStr = root:ICARTTFileFormat:Panel:IcarttPathStr
	
	NewPath/Q/o/m="Please select a folder where the icartt file will be placed." IcarttFilePath
	if (V_flag==0)	// update only if a path was selected.
		IcarttPathStr = convertpathstr("IcarttFilePath")
	endif
	
	ControlUpdate titleboxIcarttPath
	DoUpdate
	
End


// *************************************************
// Sets globals for when a user wants to edit text beyond the display window.

Function IcarttCheck_RadioComments(name, value) : CheckBoxControl
	String name
	Variable value

	SVAR RadioValStr = root:ICARTTFileFormat:RadioValStr
	
	strswitch(name)
		case  "radioPIAffiliation":
			RadioValStr = "PIAffiliationStr"
			break
		case  "radioDataSourceDescription":
			RadioValStr = "DataSourceDescriptionStr"
			break
		case  "radioSpecialComments":
			RadioValStr = "SpecialCommentsStr"
			break

		case  "radioPIContactInfo":
			RadioValStr = "PIContactInfoStr"
			break
		case  "radioLocationInfo":
			RadioValStr = "LocationInfoStr"
			break
		case  "radioInstrumentInfo":
			RadioValStr = "InstrumentInfoStr"
			break
		case  "radioDataInfo":
			RadioValStr = "DataInfoStr"
			break
		case  "radioUncertaintyInfo":
			RadioValStr = "UncertaintyinfoStr"
			break
		case  "radioStipulationsOnUse":
			RadioValStr = "StipulationsOnUseStr"
			break
		case  "radioOtherComments":
			RadioValStr = "OtherCommentsStr"
			break
		case  "radioRevisionComments":
			RadioValStr = "RevisionStr"
			break

	endswitch
		
	CheckBox radioPIAffiliation, value =  !cmpstr(RadioValStr, "PIAffiliationStr")
	CheckBox radioDataSourceDescription, value =  !cmpstr(RadioValStr, "DataSourceDescriptionStr")
	CheckBox radioSpecialComments, value =  !cmpstr(RadioValStr, "SpecialCommentsStr")

	CheckBox radioPIContactInfo, value =  !cmpstr(RadioValStr, "PIContactInfoStr")
	CheckBox radioLocationInfo, value =  !cmpstr(RadioValStr, "LocationInfoStr")
	CheckBox radioInstrumentInfo, value =  !cmpstr(RadioValStr, "InstrumentInfoStr")
	CheckBox radioDataInfo, value =  !cmpstr(RadioValStr, "DataInfoStr")
	CheckBox radioUncertaintyInfo, value =  !cmpstr(RadioValStr, "UncertaintyinfoStr")
	CheckBox radioStipulationsOnUse, value =  !cmpstr(RadioValStr, "StipulationsOnUseStr")
	CheckBox radioOtherComments, value =  !cmpstr(RadioValStr, "OtherCommentsStr")
	CheckBox radioRevisionComments, value =  !cmpstr(RadioValStr, "RevisionStr")

End


// *************************************************
// Creates and editing window for user comments and other header info.

Function IcarttButton_EditParam(ctrlName) : ButtonControl
	String ctrlName
	
	SVAR RadioValStr = root:ICARTTFileFormat:RadioValStr
	
	if (strlen(RadioValStr) > 0)
		SVAR ThisParam = $"root:ICARTTFileFormat:"+RadioValStr
		DoWindow/K EditTextBox
		NewNotebook/F=0/W=(462,145,1050,310)/k=1/n=EditTextBox as "EditTextBox"
		Notebook EditTextBox  text = ThisParam
	else
		Abort "Please click a radio button before the edit button. - Aborting from IcarttButton_EditParam"
	endif
End


// *************************************************
// Transfers the text of the editing window into the global string variable.

Function IcarttButton_AcceptParam(ctrlName) : ButtonControl
	String ctrlName

	DoWindow/F EditTextBox
	if (V_flag==0)
		Abort "The EditTextBox isn't open so nothing was accepted. - Aborting from IcarttButton_AcceptParam"
	endif

	SVAR RadioValStr = $"root:ICARTTFileFormat:RadioValStr"
	SVAR ThisParam = $"root:ICARTTFileFormat:"+RadioValStr

	if (SVAR_Exists(ThisParam)==0)
		Abort "Something is wrong with the radio value. Verify that  "+ThisParam+" was checked. - Aborting from IcarttButton_AcceptParam"		
	endif

	string CheckBoxListNoCRLF, CheckBoxListYesCRLF
	
	CheckBoxListNoCRLF=""
	CheckBoxListNoCRLF += "PIAffiliationStr;DataSourceDescriptionStr;"
	CheckBoxListNoCRLF += "PIContactInfoStr;LocationInfoStr;InstrumentInfoStr;DataInfoStr;"
	CheckBoxListNoCRLF += "UncertaintyinfoStr;StipulationsOnUseStr;"

	CheckBoxListYesCRLF = ""
	CheckBoxListYesCRLF += "SpecialCommentsStr;OtherCommentsStr;RevisionStr"  // not used, just for reference
	
	Notebook EditTextBox selection = {startofFile, endOfFile}
	GetSelection notebook, EditTextBox, 2
	
	if (FindListItem(RadioValStr, CheckBoxListNoCRLF, ";", 0) >= 0 )
		ThisParam = RemoveCarriageReturns(S_Selection)
	else
		ThisParam = RemoveTrailingCRorLF(S_Selection)	
	endif
	
	DoWindow/K EditTextBox

End


// *************************************************
// Transfers info from text wave into TextMatrix2D.  
// Called from IcarttPopup_IndepVarName.

Function Icartt_Reorganize2DTextMatrix(TextMatrix2D, Textwave)
	Wave/t TextMatrix2D, Textwave

	variable i, numRows = numpnts(Textwave)
		
	make/o/t/n=(numRows,2) NewTextMatrix2D
	
	for(i=0;i<numRows;i+=1)
		NewTextMatrix2D[i][0] = "1"
		NewTextMatrix2D[i][1] = Textwave[i]
	endfor
	
	TextMatrix2D = NewTextMatrix2D
	
	Killwaves NewTextMatrix2D

End


// *************************************************
// Provides a clean way to exit the panel for those who may be unsure.

Function IcarttButton_Exit(ctrlName) : ButtonControl
	String ctrlName

	DoWindow/K ICARTT_Panel
	DoWindow/K EditTextBox
	DoWindow/K ICARTTICARTDataMgtNotebook
	
End

